<?php
/**
 * XTT admin class
 *
 * @package Xili-Tidy-Tags
 * @subpackage core
 * @since 1.12
 */


/**
 * xili-tidy-tags class admin (now separated file since 1.8.0) - 1.8.1 fixes - 1.8.2 (s)-
 *
 * 1.8.3 - Add errors management to detect bad instancing with other tag (custom taxonomy) ...
 * 1.8.7 - Tags grouping via alias - fixes errors when no declared taxonomy and more...
 * 1.9.1 - WP 3.8 updated message
 * 1.9.2 - WP 3.9-b1 updated notice message prepare
 * 1.9.3 - 141128 - Fixes $news_id pointer in admin
 * 1.10.0 - 141219 - Fixes editing alias tags
 * 1.10.1 - 150228 - pre-tested WP 4.2-alpha - rewrite selected(), checked()
 * 1.10.2 - 150322 - new datatables JS+CSS (1.10.5)
 * 1.10.3 - 150408 - fixes notice with constante line 61
 * 1.11.0 - 150703 - datatables js 1.10.7 updated WP 43 + debug
 * 1.11.4 - 160722 - insert commits of Naja
 * 1.11.5 - 170620 - fixes group sorting
 * 1.12.0 - 190517 - Code sources rewritting with WPCS
 * 1.12.02 - 200619 - fixes links in group admin...
 * 1.12.03 - 200805 - fixes input of group tags
 */

class Xili_Tidy_Tags_Admin extends Xili_Tidy_Tags {

	public $subselect = 0; /* selected parent group */
	public $fromgroupselect = 0; /* group in which belong tags */
	public $groupexclude = false; /* exclude group from the query */
	public $uncheckedtags = false; /* exclude uncheckedtags from the query if false */
	public $onlyuncheckedtags = false; /* for general query to see only unchecked */
	public $onlyparent = false; /* when a group is parent, show all tags of childs of the group */
	public $news_id = 0; //for multi pointers

	/**
	 * PHP 5 Constructor
	 */
	public function __construct( $xtt_parent, $post_tag = 'post_tag', $post_tag_post_type = 'post' ) {

		$this->parent = $xtt_parent; // to keep values built in parent filters...

		parent::__construct( $post_tag, $post_tag_post_type, true );  // need parent constructed values (third param - tell coming admin-class //2.6
		add_action( 'admin_init', array( &$this, 'admin_init' ) ); // 1.5.0
		add_action( 'admin_menu', array( &$this, 'xili_add_pages' ) );

		add_action( $this->post_tag . '_add_form', array( &$this, 'xili_add_tag_form' ) ); /* to choose a group for a new tag */
		add_action( $this->post_tag . '_add_form_fields', array( &$this, 'xili_add_tag_alias' ) ); /* to assign a tag as alias - 1.8.7 */
		add_action( $this->post_tag . '_edit_form', array( &$this, 'xili_edit_tag_form' ) );
		add_action( $this->post_tag . '_edit_form', array( &$this, 'xili_add_tag_alias' ) ); /* to assign a tag as alias - 1.8.7 */

		/*  edit-tags table */
		add_filter( 'manage_edit-' . $this->post_tag . '_columns', array( &$this, 'xili_manage_tax_column_name' ) );
		add_filter( 'manage_' . $this->post_tag . '_custom_column', array( &$this, 'xili_manage_tax_column' ), 10, 3 ); // 2.6

		/* actions for post and page admin UI */
		add_action( 'save_post', array( &$this, 'xili_tags_grouping' ), 50 ); /* to affect tags to lang of saved post */

		add_action( 'created_term', array( &$this, 'xili_created_term' ), 9, 2 ); /* a new term was created */
		add_action( 'edited_term', array( &$this, 'xili_edited_term' ), 9, 2 ); /* a term was edited */

		/* plugin list */
		add_filter( 'plugin_action_links', array( &$this, 'xili_filter_plugin_actions' ), 100, 2 );
		/* help */
		add_action( 'contextual_help', array( &$this, 'add_help_text' ), 10, 3 ); /* 1.5.0 */
	}

	public function admin_init() {
		$suffix = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? '' : '.min';
		/* Register our script. */
		wp_register_script( 'datatables-v10', plugins_url( 'js/jquery.dataTables' . $suffix . '.js', $this->file_file ), array( 'jquery' ), '1.10.7', true );

		wp_register_style( 'table_style-v10', plugins_url( '/css/jquery.dataTables' . $suffix . '.css', $this->file_file ), array(), '1.10.7', 'screen' );
	}

	/**
	 * add admin menu and associated pages of admin UI tools page xili_tidy_editor_set
	 *
	 * @since 0.8.0
	 * @updated 0.9.5 - menu without repeat of main title, levels with new caps set by plugin array( &$this, 'top_tidy_menu_title' )
	 * @updated 1.0.1 - favicon.ico for menu title - 1.3.4 Special window for Super User
	 */
	public function xili_add_pages() {

		$taxo_tag = get_taxonomy( $this->post_tag );
		$this->tags_name = ( isset( $taxo_tag->labels->name ) ) ? $taxo_tag->labels->name : $this->post_tag;
		$this->singular_name = ( isset( $taxo_tag->labels->singular_name ) ) ? $taxo_tag->labels->singular_name : $this->post_tag; // 1.8.6

		$pre = ( 'post_tag' == $this->post_tag ) ? '' : '_' . $this->post_tag;
		/* translators: */
		$this->thehook0 = add_menu_page( sprintf( __( '%s groups', 'xili-tidy-tags' ), $this->tags_name ), sprintf( __( 'Tidy %s', 'xili-tidy-tags' ), $this->tags_name ), '', 'xili-tidy-tags' . $pre, array( &$this, 'top_tidy_menu_title' ), plugins_url( 'images/xilitidy-logo-16.png', $this->file_file ) ); //1.5.5
		/* translators: */
		$this->thehooka = add_submenu_page( 'xili-tidy-tags' . $pre, sprintf( __( '%s groups', 'xili-tidy-tags' ), $this->tags_name ), __( 'Info for SuperAdmin', 'xili-tidy-tags' ), '', 'xili-tidy-tags' . $pre, array( &$this, 'top_tidy_menu_title' ) );
		/* translators: */
		$this->thehook = add_submenu_page( 'xili-tidy-tags' . $pre, sprintf( __( '%s groups', 'xili-tidy-tags' ), $this->tags_name ), sprintf( __( 'Tidy %s settings', 'xili-tidy-tags' ), $this->tags_name ), 'xili_tidy_editor_set', 'xili_tidy_tags_settings' . $pre, array( &$this, 'xili_tidy_tags_settings' ));
		add_action( 'load-' . $this->thehook, array( &$this, 'on_load_page' ) );

		/* sub-page */
		/* translators: */
		$this->thehook2 = add_submenu_page( 'xili-tidy-tags' . $pre, sprintf( __( 'Tidy %s', 'xili-tidy-tags' ), $this->tags_name ), sprintf( __( 'Tidy %s assign', 'xili-tidy-tags' ), $this->tags_name ), 'xili_tidy_editor_group', 'xili_tidy_tags_assign' . $pre, array( &$this, 'xili_tidy_tags_assign' ));

		add_action( 'load-' . $this->thehook2, array( &$this, 'on_load_page2' ) );

		add_action( 'admin_print_scripts-' . $this->thehook2, array( &$this, 'admin_enqueue_scripts' ) );
		add_action( 'admin_print_styles-' . $this->thehook2, array( &$this, 'admin_enqueue_styles' ) );

		$this->insert_news_pointer( 'xtt_new_version' ); // pointer in menu for updated version
		add_action( 'admin_print_footer_scripts', array( &$this, 'print_the_pointers_js' ) );
		// 1.5
	}

	public function top_tidy_menu_title() {
		// again with wp3.0 instead '' call v1.3.4
		$pre = ( 'post_tag' == $this->post_tag ) ? '' : '_' . $this->post_tag;
		?>
		<div class='wrap'>
		<h2><?php printf( esc_html__( 'Tidy %s settings', 'xili-tidy-tags' ), $this->tags_name ); ?></h2>
		<h4><?php esc_html_e( 'This window is reserved for future settings in multisite mode (wpmu) for administrator like SuperAdmin... ', 'xili-tidy-tags' ); ?></h4>
		<p><?php printf( esc_html__( 'Link to set tidy %s in current site', 'xili-tidy-tags' ), $this->tags_name ); ?>: <a href="<?php echo 'admin.php?page=xili_tidy_tags_settings' . $pre; ?>" title="xili-tidy-tags settings" ><?php /* translators: */ printf( esc_html__( 'To create groups of %s', 'xili-tidy-tags' ), $this->tags_name ); ?></a></p>
		<p><?php printf( esc_html__( 'Link to assign tidy %s in current site', 'xili-tidy-tags' ), $this->tags_name ); ?>: <a href="<?php echo 'admin.php?page=xili_tidy_tags_assign' . $pre; ?>" title="xili-tidy-tags assign"><?php /* translators: */ printf( esc_html__( 'To assign a group to %s', 'xili-tidy-tags' ), $this->tags_name ); ?></a></p>

		<h4><a href="http://dev.xiligroup.com/xili-tidy-tags" title="Plugin page and docs" target="_blank" style="text-decoration:none" ><img style="vertical-align:middle" src="<?php echo plugins_url( 'images/xilitidy-logo-32.png', $this->file_file ); ?>" alt="xili-tidy-tags logo"/></a> - © <a href="http://dev.xiligroup.com" target="_blank" title="<?php esc_html_e( 'Author' ); ?>" >xiligroup.com</a>™ - msc 2009-19 - v. <?php echo XILITIDYTAGS_VER; ?></h4>
		</div>
		<?php
	}

	/* Dashboard - Manage - Tidy tags */
	public function xili_tidy_tags_settings() {
		global $wp_version;

		$xili_tidy_tags_page = 'settings';
		$formtitle = __( 'Add a group', 'xili-tidy-tags' ); /* translated in form */
		$submit_text = __( 'Add &raquo;', 'xili-tidy-tags' );
		$cancel_text = __( 'Cancel' );
		$action = '';
		$optionmessage = '';
		$emessage = '';
		$tagsgroup = null;
		$term_id = 0;
		if ( isset( $_POST['reset'] ) ) {
			$action = $_POST['reset'];
		} elseif ( isset( $_POST['updateoptions'] ) ) {
			$action = 'updateoptions';
		} elseif ( isset( $_POST['importxililanguages'] ) ) {
			$action = 'importxililanguages';
		} elseif ( isset( $_POST['importacat'] ) ) {
			$action = 'importacat';
		} elseif ( isset( $_POST['editor_caps_submit'] ) ) { /* 0.9.5 capabilities */
			$action = 'editor_caps_submit';
		} elseif ( isset( $_POST['sendmail'] ) ) { //1.5.4
			$action = 'sendmail';
		} elseif ( isset( $_POST['action'] ) ) {
			$action = $_POST['action'];
		}

		if ( isset( $_GET['action'] ) ) :
			$action = $_GET['action'];
			$term_id = $_GET['term_id'];
		endif;
		$message = $action;
		$msg = 0;
		switch ( $action ) {
			case 'editor_caps_submit':
				check_admin_referer( 'xilitagsettings' );
				$new_cap = $_POST['editor_caps'];

				$this->xili_settings['editor_caps'] = $new_cap;
				$this->xili_settings['datatable_js'] = ( isset( $_POST['datatable_js'] ) ) ? $_POST['datatable_js'] : ''; // 1.5.3.1

				update_option( 'xili_tidy_tags_settings', $this->xili_settings );

				$actiontype = 'add';
				$message .= ' - ' . __( 'Editor Capabilities changed to: ', 'xili-tidy-tags' ) . ' (' . $new_cap . ') ';
				$optionmessage = $message;
				$msg = 6;
				break;
			case 'importacat':
				check_admin_referer( 'xilitagsettings' );
				$chosencatid = $_POST['catsforgroup'];
				$chosenparent = $_POST['tags_parent'];
				$chosencat = get_category( $chosencatid );
				$desc = __( 'Group for: ', 'xili-tidy-tags' ) . $chosencat->name . ' ' . __( 'category', 'xili-tidy-tags' );
				$args = array(
					'alias_of' => '',
					'description' => $desc,
					'parent' => (int) $_POST['tags_parent'],
				);
				$theids = wp_insert_term( $chosencat->name, $this->tidy_taxonomy, $args );
				if ( ! is_wp_error( $theids ) ) {
					wp_set_object_terms( $theids['term_id'], (int) $_POST['tags_parent'], $this->tidy_taxonomy );
				}

				$actiontype = 'add';
				$message .= ' - ' . __( 'This group was added: ', 'xili-tidy-tags' ) . ' (' . $chosencatid . ') parent = ' . $chosenparent;
				break;

			case 'importxililanguages':
				check_admin_referer( 'xilitagsettings' );
				$this->xili_langs_import_terms();
				$actiontype = 'add';
				$message .= ' - ' . __( 'The languages groups was added. ', 'xili-tidy-tags' );
				$msg = 5;
				break;

			case 'add':
				check_admin_referer( 'xilitagsettings' );
				$term = $_POST['tagsgroup_name'];
				if ( '' != $term ) {
					$args = array(
						'alias_of' => '',
						'description' => $_POST['tagsgroup_description'],
						'parent' => (int) $_POST['tagsgroup_parent'],
						'slug' => $_POST['tagsgroup_nicename'],
					);
					$theids = wp_insert_term( $term, $this->tidy_taxonomy, $args );
					if ( ! is_wp_error( $theids ) ) {
						wp_set_object_terms( $theids['term_id'], (int) $_POST['tagsgroup_parent'], $this->tidy_taxonomy );
						$message .= ' - ' . __( 'A new group was added. ', 'xili-tidy-tags' );
						$msg = 1;
					} else {
						$message .= ' - Error DB: ' . serialize( $theids );
						$msg = 2;
					}
					$actiontype = 'add';

				} else {
					$actiontype = 'add';
					$message .= ' - ' . esc_html__( 'NO new group was added. ', 'xili-tidy-tags' );
					$msg = 2;
				}
				break;

			case 'edit':
				$actiontype = 'edited';
				$tagsgroup = get_term( (int) $term_id, $this->tidy_taxonomy, OBJECT, 'edit' );
				$submit_text = esc_html__( 'Update &raquo;', 'xili-tidy-tags' );
				$formtitle =  esc_html__( 'Edit Group', 'xili-tidy-tags' );
				$message .= ' - ' . esc_html__( 'Group to update. ', 'xili-tidy-tags' );
				break;

			case 'edited':
				check_admin_referer( 'xilitagsettings' );
				$actiontype = 'add';
				$term_id = $_POST['tagsgroup_term_id'];
				$term_name = $_POST['tagsgroup_name']; // fixed 1.6.0
				$args = array(
					'name' => $term_name,
					'alias_of' => '',
					'description' => $_POST['tagsgroup_description'],
					'parent' => (int)$_POST['tagsgroup_parent'],
					'slug' =>$_POST['tagsgroup_nicename'],
				);
				$theids = wp_update_term( $term_id, $this->tidy_taxonomy, $args );
				$message .= ' - ' .__( 'A group was updated. ', 'xili-tidy-tags' );
				$msg = 3;
				break;

			case 'delete':
				$actiontype = 'deleting';
				$submit_text = __( 'Delete &raquo;', 'xili-tidy-tags' );
				$formtitle = __( 'Delete group', 'xili-tidy-tags' );
				$tagsgroup = get_term( (int) $term_id, $this->tidy_taxonomy, OBJECT, 'edit' );
				$message .= ' - ' . __( 'A group to delete. ', 'xili-tidy-tags' );
				break;

			case 'deleting':
				check_admin_referer( 'xilitagsettings' );
				$actiontype = 'add';
				$term = $_POST['tagsgroup_term_id'];
				wp_delete_term( $term, $this->tidy_taxonomy ); // 1.8.1
				$message .= ' - ' . __( 'A group was deleted. ', 'xili-tidy-tags' );
				$msg = 4;
				break;

			case 'reset':
				$actiontype = 'add';
				break;

			case 'sendmail':
				// 1.5.4
				check_admin_referer( 'xilitagsettings' );
				$this->xili_settings['url'] = ( isset( $_POST['urlenable'] ) ) ? $_POST['urlenable'] : '';
				$this->xili_settings['theme'] = ( isset( $_POST['themeenable'] ) ) ? $_POST['themeenable'] : '';
				$this->xili_settings['wplang'] = ( isset( $_POST['wplangenable'] ) ) ? $_POST['wplangenable'] : '';
				$this->xili_settings['version-wp'] = ( isset( $_POST['versionenable'] ) ) ? $_POST['versionenable'] : '';
				$this->xili_settings['xiliplug'] = ( isset( $_POST['xiliplugenable'] ) ) ? $_POST['xiliplugenable'] : '';
				$this->xili_settings['webmestre-level'] = $_POST['webmestre']; // 1.8.2
				update_option( 'xili_tidy_tags_settings', $this->xili_settings );
				$contextual_arr = array();
				if ( 'enable' == $this->xili_settings['url'] ) {
					$contextual_arr[] = 'url=[ ' . get_bloginfo( 'url' ) . ' ]';
				}
				if ( isset( $_POST['onlocalhost'] ) ) {
					$contextual_arr[] = 'url=local';
				}
				if ( 'enable' == $this->xili_settings['theme'] ) {
					$contextual_arr[] = 'theme=[ ' . get_option( 'stylesheet' ) . ' ]';
				}
				if ( 'enable' == $this->xili_settings['wplang'] ) {
					$contextual_arr[] = 'WPLANG=[ ' . $this->get_wplang() . ' ]';
				}
				if ( 'enable' == $this->xili_settings['version-wp'] ) {
					$contextual_arr[] = 'WP version=[ ' . $wp_version . ' ]';
				}
				if ( 'enable' == $this->xili_settings['xiliplug'] ) {
					$contextual_arr[] = 'xiliplugins=[ ' . $this->check_other_xili_plugins() . ' ]';
				}

				$contextual_arr[] = $this->xili_settings['webmestre-level'];  // 1.9.1

				$headers = 'From: xili-tidy-tags plugin page <' . get_bloginfo( 'admin_email' ) . '>' . "\r\n";
				if ( '' != $_POST['ccmail'] ) {
					$headers .= 'Cc: <' . $_POST['ccmail'] . '>' . "\r\n";
					$headers .= 'Reply-To: <' . $_POST['ccmail'] . '>' . "\r\n";
				}
				$headers .= '\\';
				$message = 'Message sent by: ' . get_bloginfo( 'admin_email' ) . "\n\n";
				$message .= 'Subject: ' . $_POST['subject'] . "\n\n";
				$message .= 'Topic: ' . $_POST['thema'] . "\n\n";
				$message .= 'Content: ' . $_POST['mailcontent'] . "\n\n";
				$message .= 'Checked contextual infos: ' . implode( ', ', $contextual_arr ) . "\n\n";
				$message .= 'This message was sent by webmaster in xili-tidy-tags plugin settings page' . "\n\n";
				$message .= "\n\n";
				$result = wp_mail( 'contact@xiligroup.com', $_POST['thema'] . ' from xili-tidy-tags v. ' . XILITIDYTAGS_VER . ' plugin settings page. ', $message, $headers );
				$message = __( 'Email sent. ', 'xili-tidy-tags' );
				$msg = 7;
				/* translators: */
				$emessage = sprintf( __( 'Thanks for your email. A copy was sent to %s (%s)', 'xili-tidy-tags' ), $_POST['ccmail'], $result );
				$actiontype = 'add';
				break;

			default:
				$actiontype = 'add';
				/* translators: */
				$message .= sprintf( __( 'Find the list of groups for %s. ', 'xili-tidy-tags' ), $this->tags_name );
		}

		/* register the main boxes always available */
		/* translators: */
		add_meta_box( 'xili_tidy_tags-normal-group-tags-list', sprintf( __( 'Groups of %s', 'xili-tidy-tags' ), $this->tags_name ), array( &$this, 'on_normal_group_tags_list_content' ), $this->thehook, 'normal', 'core' ); /* list of groups*/

		add_meta_box( 'xili_tidy_tags-normal-form', __( 'The group', 'xili-tidy-tags' ), array( &$this, 'on_normal_group_form_content' ), $this->thehook, 'normal', 'core' ); /* the group*/
		add_meta_box( 'xili_tidy_tags-sidebox-import', __( 'Actions', 'xili-tidy-tags' ), array( &$this, 'on_sidebox_import_content' ), $this->thehook, 'side', 'core' ); /* Actions */

		$themessages[1] = __( 'A new group was added. ', 'xili-tidy-tags' );
		$themessages[2] = __( 'NO new group was added. ', 'xili-tidy-tags' );
		$themessages[3] = __( 'A group was updated. ', 'xili-tidy-tags' );
		$themessages[4] = __( 'A group was deleted. ', 'xili-tidy-tags' );
		$themessages[5] = __( 'The languages groups was added. ', 'xili-tidy-tags' );
		$themessages[6] = $optionmessage;
		$themessages[7] = __( 'Email sent. ', 'xili-tidy-tags' );

		/* form datas in array for do_meta_boxes() */
		$data = array(
			'xili_tidy_tags_page' => $xili_tidy_tags_page,
			'message' => $message,
			'action' => $action,
			'formtitle' => $formtitle,
			'tagsgroup' => $tagsgroup,
			'submit_text' => $submit_text,
			'cancel_text' => $cancel_text,
			'term_id' => $term_id,
			'emessage' => $emessage,
		);
		?>
		<div id="xili-tidy-tags-settings" class="wrap columns-2" style="min-width:880px">

			<h2><?php /* translators: */ printf( esc_html__( 'Tidy %s groups', 'xili-tidy-tags' ), $this->tags_name ); ?></h2>
		<?php
		if ( 0 != $msg ) {
			?>
			<div id="message" class="updated fade"><p><?php echo $themessages[ $msg ]; ?></p></div>
			<?php
		}
			$pre = ( 'post_tag' == $this->post_tag ) ? 'xili_tidy_tags_settings' : 'xili_tidy_tags_settings_' . $this->post_tag;
			?>
			<form name='add' id='add' method="post" action="admin.php?page=<?php echo $pre; ?>">
				<input type="hidden" name="action" value="<?php echo $actiontype; ?>" />
				<?php wp_nonce_field( 'xili-tidy-tags-settings' ); ?>
				<?php wp_nonce_field( 'closedpostboxes', 'closedpostboxesnonce', false ); ?>
				<?php
				wp_nonce_field( 'meta-box-order', 'meta-box-order-nonce', false );
				/* 0.9.3 add has-right-sidebar for next wp 2.8*/

				global $wp_version;
				if ( version_compare( $wp_version, '3.3.9', '<' ) ) {
					$poststuff_class = 'class="metabox-holder has-right-sidebar"';
					$postbody_class = '';
					$postleft_id = '';
					$postright_id = 'side-info-column';
					$postleft_class = '';
					$postright_class = 'inner-sidebar';
				} else { // 3.4
					$poststuff_class = '';
					$postbody_class = 'class="metabox-holder columns-2"';
					$postleft_id = 'id="postbox-container-2"';
					$postright_id = 'postbox-container-1';
					$postleft_class = 'class="postbox-container"';
					$postright_class = 'postbox-container';
				}
				?>
				<div id="poststuff" <?php echo $poststuff_class; ?>>
					<div id="post-body" <?php echo $postbody_class; ?> >
						<div id="<?php echo $postright_id; ?>" class="<?php echo $postright_class; ?>">
							<?php do_meta_boxes( $this->thehook, 'side', $data ); ?>
						</div>
						<div id="post-body-content">
							<div <?php echo $postleft_id; ?> <?php echo $postleft_class; ?> style="min-width:580px">
								<?php do_meta_boxes( $this->thehook, 'normal', $data ); ?>
							</div>

							<h4><a href="http://dev.xiligroup.com/xili-tidy-tags" title="Plugin page and docs" target="_blank" style="text-decoration:none" ><img style="vertical-align:middle" src="<?php echo plugins_url( 'images/xilitidy-logo-32.png', $this->file_file ); ?>" alt="xili-tidy-tags logo"/></a> - © <a href="http://dev.xiligroup.com" target="_blank" title="<?php esc_html_e( 'Author' ); ?>" >xiligroup.com</a>™ - msc 2009-19 - v. <?php echo XILITIDYTAGS_VER; ?></h4>

						</div>
					</div>
					<br class="clear" />
				</div>
				<?php wp_nonce_field( 'xilitagsettings' ); ?>
		</form>
		</div>
		<script type="text/javascript">
			//<![CDATA[
			jQuery(document).ready( function( $) {

				// close postboxes that should be closed
				$( ' .if-js-closed' ).removeClass( 'if-js-closed' ).addClass( 'closed' );
				// postboxes setup
				postboxes.add_postbox_toggles( '<?php echo $this->thehook; ?>' );
			});
			//]]>
		</script>

	<?php
	}

	public function on_load_page() {
		wp_enqueue_script( 'common' );
		wp_enqueue_script( 'wp-lists' );
		wp_enqueue_script( 'postbox' );

		$pre = ( 'post_tag' == $this->post_tag ) ? '' : $this->post_tag . '_';
		add_meta_box( 'xili_tidy_tags-sidebox-1', __( 'Message', 'xili-tidy-tags' ), array( &$this, 'on_sidebox_msg_content' ), $this->thehook, 'side', 'core' );
		add_meta_box( $pre . 'xili_tidy_tags-sidebox-3', __( 'Info', 'xili-tidy-tags' ), array( &$this, 'on_sidebox_info_content' ), $this->thehook, 'side', 'core' );
		add_meta_box( $pre . 'xili_tidy_tags-sidebox-mail', __( 'Mail & Support', 'xili-tidy-tags' ), array( &$this, 'on_sidebox_mail_content' ), $this->thehook, 'normal', 'low' );
		if ( current_user_can( 'xili_tidy_admin_set' ) ) {
			add_meta_box( $pre . 'xili_tidy_tags-sidebox-4', __( 'Capabilities', 'xili-tidy-tags' ), array( &$this, 'on_sidebox_admin_content' ), $this->thehook, 'side', 'core' );
		}

	}

	/* page for tags assign to (a) group(s) */
	public function xili_tidy_tags_assign() {

		$current_taxonomy = get_taxonomy( $this->post_tag );
		if ( false == $current_taxonomy || ! in_array( $this->post_tag_post_type, $current_taxonomy->object_type ) ) {
			$msg = 3;
		}

		$action = '';
		$msg = '';
		$term_id = '';
		$tagsnamelike = '';
		$tagsnamesearch = '';
		$xili_tidy_tags_page = 'assign';
		$submit_text = __( 'Update', 'xili-tidy-tags' );
		$cancel_text = __( 'Cancel' );
		$tagsnamelike = ( isset( $_POST['tagsnamelike'] ) ) ? $_POST['tagsnamelike'] : '';
		$tagsnamesearch = ( isset( $_POST['tagsnamesearch'] ) ) ? $_POST['tagsnamesearch'] : '';
		if ( isset( $_POST['update'] ) ) {
			$action = 'update';
		}
		/* since 1.3.0 */
		if ( isset( $_POST['tagsgroup_from_select'] ) && 'no_select' != $_POST['tagsgroup_from_select'] ) {
			$this->fromgroupselect = (int) $_POST['tagsgroup_from_select'];
		} elseif ( isset( $_GET['tps'] ) && 'no_select' != $_GET['tps'] ) {
			$this->fromgroupselect = (int) $_GET['tps'];
		} else {
			$this->fromgroupselect = 0;
		}

		$this->onlyuncheckedtags = ( isset( $_POST['tagsgroup_from_select'] ) && 'onlyuncheckedtags' == $_POST['tagsgroup_from_select'] ) ? true : false;
		$this->onlyparent = ( isset( $_POST['xili_group_only_parent'] ) && 'onlyparent' == $_POST['xili_group_only_parent'] ) ? true : false;
		$this->groupexclude = ( isset( $_POST['xili_group_not_select'] ) && 'not' == $_POST['xili_group_not_select'] ) ? true : false;
		$this->uncheckedtags = ( isset( $_POST['xili_uncheckedtags'] ) && 'include' == $_POST['xili_uncheckedtags'] ) ? true : false;
		$subselectgroups = array();
		if ( isset( $_POST['tagsgroup_parent_select'] ) && 'no_select'  != $_POST['tagsgroup_parent_select'] ) {
			$this->subselect = (int) $_POST['tagsgroup_parent_select'];
		} else {
			$this->subselect = 0;
		}

		if ( isset( $_GET['tps'] ) ) { // 1.7
			$action = 'subselectiong';
		}
		if ( isset( $_POST['subselection'] ) ) {
			$action = 'subselection';
		}
		if ( isset( $_POST['notagssublist'] ) ) {
			$action = 'notagssublist';
		}
		if ( isset( $_POST['tagssublist'] ) ) {
			$action = 'tagssublist';
		}
		if ( isset( $_GET['action'] ) ) :
			$action = $_GET['action'];
			$term_id = $_GET['term_id'];
		endif;
		$message = $action;
		switch ( $action ) {

			case 'notagssublist':
				$tagsnamelike = '';
				$tagsnamesearch = '';
				$this->fromgroupselect = 0; /* since 1.3.0 */
				$this->groupexclude = false;
				$message .= ' no sub list of tags';
				$actiontype = 'add';
				break;

			case 'tagssublist':
				check_admin_referer( 'xilitagassign' );
				$message .= ' sub list of tags starting with ' . $_POST['tagsnamelike'];
				$message .= ' . From group ' . $_POST['tagsgroup_from_select'];
				$actiontype = 'add';
				break;

			case 'subselection':
				check_admin_referer( 'xilitagassign' );
				$tagsnamelike = $_POST['tagsnamelike'];
				$tagsnamesearch = $_POST['tagsnamesearch'];
				$message .= ' selection of ' . $_POST['tagsgroup_parent_select'];
				$msg = 2;
				$actiontype = 'add';
				break;

			case 'subselectiong': // 1.7
				check_admin_referer( 'xtt-tps' );
				$message .= ' selection of ' . $_GET['tps'];
				$msg = 2;
				$actiontype = 'add';
				break;

			case 'update':
				check_admin_referer( 'xilitagassign' );
				$message .= ' ok: datas are saved... ';
				$message .= $this->checkboxes_update_them( $tagsnamelike, $tagsnamesearch );
				$msg = 1;
				$actiontype = 'add';
				break;

			case 'reset':
				$actiontype = 'add';
				break;

			default:
				$actiontype = 'add';
				/* translators: */
				$message .= sprintf( __( 'Find the list of %s. ', 'xili-tidy-tags' ), $this->tags_name );
		}
		/* form datas in array for do_meta_boxes() */
		$data = array(
			'xili_tidy_tags_page' => $xili_tidy_tags_page,
			'message' => $message,
			'action' => $action,
			'submit_text' => $submit_text,
			'cancel_text' => $cancel_text,
			'term_id' => $term_id,
			'tagsnamesearch' => $tagsnamesearch,
			'tagsnamelike' => $tagsnamelike,
		);

		/* register the main boxes always available */
		add_meta_box( 'xili_tidy_tags-sidebox-action', __( 'Actions', 'xili-tidy-tags' ), array( &$this, 'on_sub_sidebox_action_content' ), $this->thehook2, 'side', 'core' ); /* Actions */
		/* translators: */
		add_meta_box( 'xili_tidy_tags-normal-tags', sprintf( esc_html__( 'Tidy %s', 'xili-tidy-tags' ), $this->tags_name ), array( &$this, 'on_sub_normal_tags_list_content' ), $this->thehook2, 'normal', 'core' ); /* list of tags*/

		$themessages[1] = __( 'List updated in database !', 'xili-tidy-tags' );

		$themessages[2] = $message;
		$themessages[3] = __( 'Post Type <strong>not declared</strong> in Custom Taxonomy !', 'xili-tidy-tags' );
			?>
		<div id="xili-tidy-tags-assign" class="wrap columns-2" style="min-width:880px">

			<h2><?php /* translators: */ printf( esc_html__( '%1$s in group with post type named: %2$s', 'xili-tidy-tags' ), $this->tags_name, $this->post_tag_post_type); ?></h2>
			<?php
			if ( 0 != $msg && '' != $msg ) {
				?>
			<div id="message" class="updated fade"><p><?php echo $themessages[ $msg ]; ?></p></div>
			<?php
			}
			$preassign = ( 'post_tag' == $this->post_tag ) ? 'xili_tidy_tags_assign' : 'xili_tidy_tags_assign_' . $this->post_tag;
			?>
			<form name='add' id='add' method="post" action="admin.php?page=<?php echo $preassign; ?>">
				<input type="hidden" name="action" value="<?php echo $actiontype; ?>" />
				<?php wp_nonce_field( 'xili-tidy-tags-settings' ); ?>
				<?php wp_nonce_field( 'closedpostboxes', 'closedpostboxesnonce', false ); ?>
				<?php
				wp_nonce_field( 'meta-box-order', 'meta-box-order-nonce', false );

				global $wp_version;
				if ( version_compare( $wp_version, '3.3.9', '<' ) ) {
					$poststuff_class = 'class="metabox-holder has-right-sidebar"';
					$postbody_class = '';
					$postleft_id = '';
					$postright_id = 'side-info-column';
					$postleft_class = '';
					$postright_class = 'inner-sidebar';
				} else { // 3.4
					$poststuff_class = '';
					$postbody_class = 'class="metabox-holder columns-2"';
					$postleft_id = 'id="postbox-container-2"';
					$postright_id = 'postbox-container-1';
					$postleft_class = 'class="postbox-container"';
					$postright_class = 'postbox-container';
				}

				?>
				<div id="poststuff" <?php echo $poststuff_class; ?>>
					<div id="post-body" <?php echo $postbody_class; ?> >
						<div id="<?php echo $postright_id; ?>" class="<?php echo $postright_class; ?>">
							<?php do_meta_boxes( $this->thehook2, 'side', $data); ?>
						</div>

						<div id="post-body-content" >
							<div <?php echo $postleft_id; ?> <?php echo $postleft_class; ?> style="min-width:580px">
								<?php do_meta_boxes( $this->thehook2, 'normal', $data ); ?>
							</div>

							<h4><a href="http://dev.xiligroup.com/xili-tidy-tags" title="Plugin page and docs" target="_blank" style="text-decoration:none" ><img style="vertical-align:middle" src="<?php echo plugins_url( 'images/xilitidy-logo-32.png', $this->file_file ); ?>" alt="xili-tidy-tags logo"/></a> - © <a href="http://dev.xiligroup.com" target="_blank" title="<?php esc_html_e( 'Author' ); ?>" >xiligroup.com</a>™ - msc 2009-19 - v. <?php echo XILITIDYTAGS_VER; ?></h4>

						</div>
					</div>
					<br class="clear" />
				</div>
				<?php wp_nonce_field( 'xilitagassign' ); ?>
		</form>
		</div>
		<script type="text/javascript">
			//<![CDATA[


			var assignTable;
			jQuery(document) .ready( function( $) {
				<?php if ( '' == $this->xili_settings['datatable_js'] ) { ?>
				$( '#tableupdating' ).hide();

				var assignTable = $( '#assigntable' ).dataTable( {
					"iDisplayLength": 30,
					"bStateSave": true,
					"sDom": '<"topbanner"ipf>rt<"bottombanner"lp><"clear">',
					"sPaginationType": "full_numbers",
					"aLengthMenu": [[15, 30, 60, -1], [15, 30, 60, "<?php esc_html_e( 'All lines', 'xili-tidy-tags' ); ?>"]],
					"oLanguage": {
						"oPaginate": {
							"sFirst": "<?php esc_html_e( 'First', 'xili-tidy-tags' ); ?>",
							"sLast": "<?php esc_html_e( 'Last page', 'xili-tidy-tags' ); ?>",
							"sNext": "<?php esc_html_e( 'Next', 'xili-tidy-tags' ); ?>",
							"sPrevious": "<?php esc_html_e( 'Previous', 'xili-tidy-tags' ); ?>"
						},
						"sInfo": "<?php printf( esc_html__( 'Showing (_START_ to _END_) of _TOTAL_ %s', 'xili-tidy-tags' ), $this->tags_name ); ?>",
						"sInfoFiltered": "<?php esc_html_e( '(filtered from _MAX_ total entries)', 'xili-tidy-tags' ); ?>",
						"sLengthMenu": "<?php esc_html_e( 'Show _MENU_ tags', 'xili-tidy-tags' ); ?>",
						"sSearch": "<?php esc_html_e( 'Filter tags:', 'xili-tidy-tags' ); ?>"
					},
					"aaSorting": [[1,'asc']],
					"aoColumns": [
						{ "bSearchable": false },
						null,
						{ "bSortable": false, "bSearchable": false },
						{ "bSortable": false, "bSearchable": false }]
				} );

				$( '#assigntable' ).css({ visibility:'visible' });

				$( '#update' ).click( function () {

					$( '#assigntable' ).hide();

					$( '#tableupdating' ).html("<br /><br /><h1><?php esc_html_e( 'Updating table of tags !', 'xili-tidy-tags' ); ?></h1>");
					$( '#tableupdating' ).show();
					assignTable.fnDestroy();
				} );
				<?php } ?>
				// close postboxes that should be closed
				$( ' .if-js-closed' ).removeClass( 'if-js-closed' ) .addClass( 'closed' );
				// postboxes setup
				postboxes.add_postbox_toggles( '<?php echo $this->thehook2; ?>' );
			});

						//]]>
		</script>
		<?php
	}

	/*
	 * Update the relationships according CheckBoxes
	 *
	 */
	public function checkboxes_update_them( $tagsnamelike = '', $tagsnamesearch = '' ) {

		$listgroups = get_terms(
			$this->tidy_taxonomy,
			array(
				'hide_empty' => false,
				'get' => 'all',
			)
		);

		if ( 0 == $this->fromgroupselect && false === $this->onlyuncheckedtags ) {
			$listtags = get_terms(
				$this->post_tag,
				array(
					'hide_empty' => false,
					'get' => 'all',
					'name__like' => $tagsnamelike,
					'search' => $tagsnamesearch,
				)
			);
		} else {
			/* since 1.3.0 */
			if ( false === $this->onlyuncheckedtags ) { // one group
				$group_id[] = $this->fromgroupselect;
				if ( false === $this->onlyparent ) {
					$childterms = get_terms(
						$this->tidy_taxonomy,
						array(
							'hide_empty' => false,
							'parent' => $this->fromgroupselect,
						)
					);
					if ( ! empty( $childterms ) ) {
						foreach ( $childterms as $childterm ) { /* if group is a parent, add all childs */
							$group_id[] = $childterm->term_id;
						}
					}
				}
			} else { // only all unchecked
				$listgroups = get_terms(
					$this->tidy_taxonomy,
					array(
						'hide_empty' => false,
						'get' => 'all',
					)
				);
				foreach ( $listgroups as $group ) {
					$group_id[] = $group->term_id;
				}
				$this->groupexclude = true;
				$this->uncheckedtags = true;
			}
			$listtags = xtt_get_terms_of_groups_new(
				$group_id,
				$this->tidy_taxonomy,
				$this->post_tag,
				array(
					'hide_empty' => false,
					'get' => 'all',
					'name__like' => $tagsnamelike,
					'search' => $tagsnamesearch,
					'orderby' => 'name',
				),
				$this->groupexclude,
				$this->uncheckedtags
			);
		}

		foreach ( $listtags as $tag ) {
			$groupids = array();
			foreach ( $listgroups as $group ) {
				$idcheck = 'line-' . $tag->term_id . '-' . $group->term_id;
				if ( isset( $_POST[ $idcheck ] ) ) {
					$groupids[] = (int) $group->term_id;
				}
			}
			$datavisible = 'hidden_termid-' . $tag->term_id;
			// if ( isset( $_POST[$datavisible] ) )
				wp_set_object_terms( (int) $tag->term_id, $groupids, $this->tidy_taxonomy, false );
		}

		return;
	}

	public function on_load_page2() {
			wp_enqueue_script( 'common' );
			wp_enqueue_script( 'wp-lists' );
			wp_enqueue_script( 'postbox' );

			$pre = ( 'post_tag' == $this->post_tag ) ? '' : $this->post_tag . '_';
			add_meta_box( $pre . 'xili_tidy_tags-sidebox-1', __( 'Message', 'xili-tidy-tags' ), array( &$this, 'on_sidebox_msg_content' ), $this->thehook2, 'side', 'core' );
			add_meta_box( $pre . 'xili_tidy_tags-sidebox-3', __( 'Info', 'xili-tidy-tags' ), array( &$this, 'on_sidebox_info_content' ), $this->thehook2, 'side', 'core' );
	}

	/**
	 * @since 1.5.0
	 * @updated 2.10.3
	 */
	public function admin_enqueue_scripts() {
		wp_enqueue_script( 'datatables-v10' );
	}

	public function admin_enqueue_styles() {
		wp_enqueue_style( 'table_style-v10' );
	}


	/******************************* TAXONOMIES LIST ****************************/

	/**
	 * add in edit list language column
	 *
	 * @since 1.7
	 *
	 *
	 */
	public function xili_manage_tax_column_name( $cols ) {

		//if ( class_exists ( 'xili_language' ) ) {
		$ends = array( 'posts' );
		$end = array();
		foreach ( $cols as $k => $v ) {
			if ( in_array( $k, $ends ) ) {
				$end[ $k ] = $v;
				unset( $cols[ $k ] );
			}
		}
		if ( defined( 'TAXONAME' ) ) {
			$cols[ TAXONAME ] = esc_html__( 'Language', 'xili-language' );
		}
		// for grouping
		$cols['alias_of'] = esc_html__( 'Alias of', 'xili-tidy-tags' );
		$cols = array_merge( $cols, $end );

			///$this->local_theme_mos = $this->get_localmos_from_theme();
		//}
		return $cols;
	}

	public function xili_manage_tax_column( $dummy, $name, $id ) {

		global $wpdb, $taxonomy;
		$tax = get_term( (int) $id, $taxonomy );
		$a = '';

		if ( defined( 'TAXONAME' ) && TAXONAME == $name ) {
			// loop of languages
			$listlanguages = get_terms( TAXONAME, array( 'hide_empty' => false ) );

			foreach ( $listlanguages as $lang ) {

				// test in group xx_yy-tidy-languages-group
				if ( is_object_in_term( $id, $this->tidy_taxonomy, $lang->slug . '-' . $this->xili_settings['tidylangsgroup'] ) ) {
					// link + class
					$group = term_exists( $lang->slug . '-' . $this->xili_settings['tidylangsgroup'], $this->tidy_taxonomy );
					$nonce_url = wp_nonce_url( 'admin.php?page=xili_tidy_tags_assign&tps=' . $group['term_id'], 'xtt-tps' ); // wp-admin/admin.php?page=xili_tidy_tags_assign&tps=8
					/* translators: */
					$a .= '<span title="' . sprintf( esc_html__( 'Tags in %s. ', 'xili-tidy-tags' ), $lang->description ) . '" class="curlang lang-' . $lang->slug . '"><a href="' . $nonce_url . '" >' . $lang->name . '</a></span>';
				}
			}
			if ( '' != $a ) {
				$a = '<div class="edittag" >' . __( 'assigned in:', 'xili-tidy-tags' ) . ' ' . $a;
				$a .= '</div>';
				return $a;
			}
		} else if ( 'alias_of' == $name ) {

			if ( $tax->term_group > 0 ) {
				// search terms of same group
				$alias_group = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM $wpdb->terms WHERE term_group = %s", $tax->term_group) );

				if ( $alias_group ) {
					foreach ( $alias_group as $one_alias ) {
						if ( $one_alias->slug != $tax->slug ) {
							$link = get_edit_tag_link( $one_alias->term_id, $taxonomy );
							$a .= '<a href="' . $link . '">' . $one_alias->name . '</a>';
							//if ( class_exists( 'xili_language' ) ) {
							if ( $langs = $this->return_lang_of_tag( $one_alias->term_id ) ) {
								foreach ( $langs as $lang ) {
									/* translators: */
									$a .= ' <span title="' . sprintf( esc_html__( 'Tags in %s. ', 'xili-tidy-tags' ), $lang->description ) . '" class="curlang lang-' . $lang->slug . '"><a href="' . $link . '">' . $lang->name . '</a></span>';
								}
							}
							//}
							$a .= '<br />';
						}
					}
				}
				if ( '' != $a ) {
					$a = '<div class="edittag" ><em>' . __( 'Alias:', 'xili-tidy-tags' ) . '</em><br />' . $a;
					$a .= '</div>';
					return $a;
				}
			}
			return $a;
		}
	}


	/**
	 * in post edit UI if new term was created - give it a group
	 *
	 * @since 0.9.4
	 *
	 *
	 */
	public function xili_tags_grouping( $post_ID ) {

		if ( ! class_exists( 'xili_language' ) ) {
			return; /* only used if present */
		}

		$list_tags = wp_get_object_terms( $post_ID, $this->post_tag );
		if ( ! $list_tags ) {
			return; /* no tag*/
		}
		$post_curlang = get_cur_language( $post_ID );

		$listlanggroups = get_terms(
			$this->tidy_taxonomy,
			array(
				'hide_empty' => false,
				'parent' => $this->langgroupid,
			)
		);
		if ( $listlanggroups ) {
			foreach ( $listlanggroups as $curgroup ) {
				$langsgroup[] = $curgroup->term_id;
			}
			$langsgroup[] = $this->langgroupid; /* add group parent */
			foreach ( $list_tags as $tag ) {
				/* test if the tag is owned by a group */
				$nbchecked = false;
				foreach ( $langsgroup as $onelanggroup ) {
					if ( is_object_in_term( $tag->term_id, $this->tidy_taxonomy, $onelanggroup ) ) {
						$nbchecked = true;
					}
				}
				if ( false == $nbchecked ) {
					if ( false == $post_curlang ) { /* add to group parent */
						wp_set_object_terms( (int) $tag->term_id, (int) $this->langgroupid, $this->tidy_taxonomy, false );
					} else {
						$res = term_exists( $post_curlang, $this->tidy_taxonomy );
						wp_set_object_terms( (int) $tag->term_id, (int) $res ['term_id'], $this->tidy_taxonomy, false );
					}
				}
			}
		}
	}

	/**
	 * a new term was created - grouping is assigned
	 *
	 * @since 0.8.0
	 * @updated 1.2.1, 1.10 - see xili_edited_term
	 *
	 */
	public function xili_created_term( $term_id, $tt_id ) {
		/* check if it is a term from $this->post_tag  */
		if ( ! isset( $_POST['_inline_edit'] ) ) { /* to avoid delete relationship when in quick_edit (edit-tags.php) */

			$term = get_term( (int) $term_id, $this->post_tag );
			if ( $term ) {
				$listgroups = get_terms(
					$this->tidy_taxonomy,
					array(
						'hide_empty' => false,
						'get' => 'all',
					)
				);
				$groupids = array();
				foreach ( $listgroups as $group ) {
					$idcheck = 'group-' . $group->term_id;
					if ( isset( $_POST[ $idcheck ] ) ) {
						$groupids[] = (int) $group->term_id;
					}
				}
				wp_set_object_terms( $term_id, $groupids, $this->tidy_taxonomy, false );
			}
		}
	}


	/**
	 * a new term was edited - grouping is assigned
	 *
	 * @since 1.10
	 *
	 */
	public function xili_edited_term( $term_id, $tt_id ) {
		// to avoid calls when aliases are updated
		if ( isset( $_POST['action'] ) && 'add-tag' != $_POST['action'] ) {
			return; // when called in add tag screen
		}

		if ( isset( $_POST['tag_ID'] ) && $_POST['tag_ID'] != $term_id ) {
			return; // when called in edit tag screen (modify alias)
		}

		/* check if it is a term from $this->post_tag  */
		if ( ! isset( $_POST['_inline_edit'] ) ) { /* to avoid delete relationship when in quick_edit (edit-tags.php) */

			$term = get_term( (int) $term_id, $this->post_tag );
			if ( $term ) {
				$listgroups = get_terms(
					$this->tidy_taxonomy,
					array(
						'hide_empty' => false,
						'get' => 'all',
					)
				);
				$groupids = array();
				foreach ( $listgroups as $group ) {
					$idcheck = 'group-' . $group->term_id;
					if ( isset( $_POST[ $idcheck ] ) ) {
						$groupids[] = (int) $group->term_id;
					}
				}
				wp_set_object_terms( $term_id, $groupids, $this->tidy_taxonomy, false );
			}
		}
	}


	/**
	 * add in new tag form to choose a group for a new tag
	 *
	 * @since 0.8.0
	 * @update 1.6.2 - sorted display
	 *
	 */
	public function xili_add_tag_form() {
		$listtagsgroups = get_terms(
			$this->tidy_taxonomy,
			array(
				'hide_empty' => false,
				'get' => 'all',
			)
		);
		if ( ! is_wp_error( $listtagsgroups ) && $listtagsgroups ) {
			$listtagsgroupssorted = $this->create_taggrouplist_sorted( walk_TagGroupList_sorted( $listtagsgroups, 3, null, null ) );

			$checkline = '';
			$i = 0;
			foreach ( $listtagsgroupssorted as $group ) {
				if ( 0 != $group->parent ) {
					$disp_group = $group->name;
					$checkline .= '&nbsp;-&nbsp;';
				} else {
					if ( $group->slug == $this->xili_settings['tidylangsgroup'] ) {
						$lang_class = true;
					} else {
						$lang_class = false;
					}
					$disp_group = '<strong>' . $group->name . '</strong>';
					if ( 0 != $i ) {
						$checkline .= '</br>';
					}
				}
				if ( $lang_class ) {
					 $the_class = 'class="curlang lang-' . str_replace( '-' . $this->xili_settings['tidylangsgroup'], '', $group->slug ) . '"';
					 $disp_group = '<span ' . $the_class . ' title="' . $group->name . '" >' . $disp_group . '</span>';
				}
				$checkline .= '<input type="checkbox" id="group-' . $group->term_id . '" name="group-' . $group->term_id . '" value="' . $group->term_id . '" />' . $disp_group;
				$i++;
			}
			/* translators: */
			$checkline .= '<br /><br /><small>' . sprintf( esc_html__( '© by xili-tidy-tags v. %s', 'xili-tidy-tags' ), XILITIDYTAGS_VER ) . '</small>';
			/* translators: */
			echo '<div id="xtt-edit-tag" style="max-width:525px; margin:2px 22px 2px 2px; padding:3px; border:1px solid #ccc;"><label>' . sprintf( __( '%s groups', 'xili-tidy-tags' ), $this->tags_name ) . ':</label><br />' . $checkline . '</div>';
		} else {
			echo '<p style="color:red"> ERROR IN xili-tidy-tags TAXONOMY: ' . $this->tidy_taxonomy . '</p>'; // 1.8.3
		}

	}


	/**
	 * add in edit tag form to choose a group for a edited tag
	 *
	 * @since 0.8.0
	 *
	 *
	 */
	public function xili_add_tag_alias( $tag ) {
		global $wpdb;
		$checkline = '';

		$val = ( is_string( $tag ) ) ? __( 'new', 'xili-tidy-tags' ) . ' ' : '';

		$context = ( class_exists( 'xili_language' ) ) ? __( 'in another language', 'xili-tidy-tags' ) : __( 'as synonym', 'xili-tidy-tags' );
		/* translators: */
		$checkline = '<p>' . sprintf( esc_html__( 'This sub-form contains a field to define alias of this %1$s%2$s. Choose a %2$s already present %3$s. ', 'xili-tidy-tags' ), $val, $this->singular_name, $context ) . '</p>';

		if ( ! is_string( $tag ) && $tag->term_group > 0 ) {

			$alias_group = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM $wpdb->terms WHERE term_group = %s", $tag->term_group ) );

			if ( $alias_group ) {
				/* translators: */
				$checkline .= '<p><em>' . sprintf( __( 'This %1$s (%2$s) is already present in the group %3$s:', 'xili-tidy-tags' ), $this->singular_name, $tag->name, _n( sprintf( _x( 'of the following %s', 'singular', 'xili-tidy-tags' ), $this->singular_name ), sprintf( _x( 'of the following %s', 'plural', 'xili-tidy-tags' ), $this->tags_name ), count( $alias_group ) - 1 ) ) . '</em></p>';
				$list = array();
				foreach ( $alias_group as $one_alias ) {
					if ( $one_alias->slug != $tag->slug ) {

						$list[] = $one_alias->name;
					}
				}
				$checkline .= implode( ' - ', $list );
				$checkline .= '<br /><br />';
				/* translators: */
				$checkline .= '<p><em>' . sprintf( __( 'Select below if you want to change the group of this %1$s (%2$s) . Be careful!', 'xili-tidy-tags' ), $this->singular_name, $tag->name ) . '</em></p>';
			}
		}

		$checkline .= '<select name="alias_of" id="alias_of">'; // to be used when saved id= new field

		// check another tags
		$tags = get_terms(
			$this->post_tag,
			array(
				'hide_empty' => false,
				'get' => 'all',
			)
		);
		if ( ! is_wp_error( $tags ) ) {
			$checkline .= '<option value="" >' . __( 'Choice alias... ', 'xili-tidy-tags' ) . '</option>';
			foreach ( $tags as $one_tag ) {

				if ( class_exists( 'xili_language' ) ) {

					$langs = $this->return_lang_of_tag( $one_tag->term_id );

					if ( $langs ) {
						$listnames  = array();
						foreach ( $langs as $onelang ) {
							$listnames[] = $onelang->name;
						}
						$lang_line = ' [' . implode( ' | ', $listnames ) . ']';

					} else {
						$lang_line = '';
					}
				} else {
					$lang_line = '';
				}

				$checkline .= '<option value="' . $one_tag->slug . '">' . $one_tag->name . $lang_line . '</option>';
			}

			$checkline .= '</select>';
		}
		/* translators: */
		$checkline .= '<br /><br /><small>' . sprintf( esc_html__( '© by xili-tidy-tags v. %s', 'xili-tidy-tags' ), XILITIDYTAGS_VER ) . '</small>';
		/* translators: */
		echo '<div id="xtt-edit-tag" style="max-width:525px; margin:2px 22px 2px 2px; padding:3px; border:1px solid #ccc;"><label>' . sprintf( __( 'The %s aliases group', 'xili-tidy-tags' ), $this->tags_name ) . ':<br /></label><br />' . $checkline . '</div>';

	}


	/**
	 * add in edit tag form to choose a group for a edited tag
	 *
	 * @since 0.8.0
	 *
	 *
	 */
	public function xili_edit_tag_form( $tag ) {
		$listtagsgroups = get_terms(
			$this->tidy_taxonomy,
			array(
				'hide_empty' => false,
				'get' => 'all',
			)
		);

		if ( ! is_wp_error( $listtagsgroups ) && $listtagsgroups ) {
			$listtagsgroupssorted = $this->create_taggrouplist_sorted( walk_TagGroupList_sorted( $listtagsgroups, 3, null, null ) );

			$checkline = '';
			$i = 0;
			foreach ( $listtagsgroupssorted as $group ) {
				/* add checked="checked" */
				if ( is_object_in_term( $tag->term_id, $this->tidy_taxonomy, (int) $group->term_id ) ) {
						$checked = 'checked="checked"';
				} else {
						$checked = '';
				}
				if ( 0 != $group->parent ) {
					$disp_group = $group->name;
					$checkline .= '&nbsp;-&nbsp;';
				} else {

					if ( $group->slug == $this->xili_settings['tidylangsgroup'] ) {
						$lang_class = true;
					} else {
						$lang_class = false;
					}
					$disp_group = '<strong>' . $group->name . '</strong>';
					if ( 0 != $i ) {
						$checkline .= '</br>';
					}
				}
				if ( $lang_class ) {
					 $the_class = 'class="curlang lang-' . str_replace( '-' . $this->xili_settings['tidylangsgroup'], '', $group->slug ) . '"';
					 $disp_group = '<span ' . $the_class . ' title="' . $group->name . '" >' . $disp_group . '</span>';
				}
				$checkline .= '<input type="checkbox" name="group-' . $group->term_id . '" id="group-' . $group->term_id . '" value="' . $group->term_id . '" ' . $checked . ' />' . $disp_group;
				$i++;
			}
			/* translators: */
			$checkline .= '<br /><br /><small>' . sprintf( esc_html__( '© by xili-tidy-tags v. %s', 'xili-tidy-tags' ), XILITIDYTAGS_VER ) . '</small>';
			// display
			/* translators: */
			echo '<div id="xtt-edit-tag" style="width:525px; margin:2px; padding:3px; border:1px solid #ccc;"><label>' . sprintf( __( '%s groups', 'xili-tidy-tags' ), $this->tags_name ) . ':<br /></label><br />' . $checkline . '</div>';

		} else {
			echo '<p style="color:red"> ERROR IN xili-tidy-tags TAXONOMY: ' . $this->tidy_taxonomy . '</p>'; // 1.8.3
			/* translators: */
			echo '<br /><br /><small>' . sprintf( esc_html__( '© by xili-tidy-tags v. %s', 'xili-tidy-tags' ), XILITIDYTAGS_VER ) . '</small>';
		}
	}




	/**
	 * Add action link(s) to plugins page
	 *
	 * @since 0.8.0
	 * @author MS
	 * @copyright Dion Hulse, http://dd32.id.au/wordpress-plugins/?configure-link and scripts@schloebe.de
	 *
	 * @updated 1.9 - fixes link when instancing...
	 */
	public function xili_filter_plugin_actions( $links, $file ) {
		static $this_plugin;

		if ( ! $this_plugin ) {
			$this_plugin = plugin_basename( $this->file_file );
		}

		if ( $file == $this_plugin ) {
			$suffix = ( 'post_tag' == $this->post_tag ) ? '' : '_' . $this->post_tag;
			$title = ( 'post_tag' == $this->post_tag ) ? __( 'Settings' ) : __( 'Settings' ) . ': ' . $this->post_tag;
			$settings_link = '<a title="' . esc_attr( $title ) . '" href="admin.php?page=xili_tidy_tags_settings' . $suffix . '">' . __( 'Settings' ) . '</a>';
			$links = array_merge( array( $settings_link ), $links ); // before other links
		}
		return $links;
	}

	/**
	 * Contextual help
	 *
	 * @since 1.5.0
	 */
	public function add_help_text( $contextual_help, $screen_id, $screen ) {

		if ( false !== strpos( $screen->id, 'xili_tidy_tags_settings' ) ) {
			$to_remember =
			'<p>' . __( 'Things to remember to set xili-tidy-tags:', 'xili-tidy-tags' ) . '</p>' .
			'<ul>' .
			'<li>' . __( 'If you use it for multilingual website: verify that the xili-language trilogy is active. ', 'xili-tidy-tags' ) . '</li>' .
			'<li>' . __( 'Update the list of targeted languages. See Actions left box. ', 'xili-tidy-tags' ) . '</li>' .
			'<li>' . __( 'For current site with lot of tags, create group or sub-group and go in assign page. ', 'xili-tidy-tags' ) . '</li>' .
			'<li>' . __( 'Don’t forget to activate a tags cloud widget. ', 'xili-tidy-tags' ) . '</li>' .
			'<li>' . __( 'In widget: the group for displaying tags in current lang is the_curlang ', 'xili-tidy-tags' ) . '</li>' .
			'<li>' . __( 'In widget: the group to set is the slug - trademark for TradeMark group - if you create one. ', 'xili-tidy-tags' ) . '</li>' .
			'</ul>';

			$more_infos =
			'<p><strong>' . __( 'For more information:' ) . '</strong></p>' .
			'<p>' . __( '<a href="http://dev.xiligroup.com/xili-tidy-tags" target="_blank">Xili-tidy-tags Plugin Documentation</a>', 'xili-tidy-tags' ) . '</p>' .
			'<p>' . __( '<a href="http://codex.wordpress.org/" target="_blank">WordPress Documentation</a>', 'xili-tidy-tags' ) . '</p>' .
			'<p>' . __( '<a href="https://wordpress.org/support/plugin/xili-tidy-tags" target="_blank">Support Forums</a>', 'xili-tidy-tags' ) . '</p>';

			$screen->add_help_tab(
				array(
					'id' => 'to-remember',
					'title' => __( 'Things to remember', 'xili-tidy-tags' ),
					'content' => $to_remember,
				)
			);

			$screen->add_help_tab(
				array(
					'id' => 'more-infos',
					'title'   => __( 'For more information', 'xili-tidy-tags' ),
					'content' => $more_infos,
				)
			);

		} else if ( false !== strpos( $screen->id, 'xili_tidy_tags_assign' ) ) {
			$to_remember =
			'<p>' . __( 'Things to remember to assign tags to groups:', 'xili-tidy-tags' ) . '</p>' .
			'<ul>' .
			'<li>' . __( 'Use features of paginated table. ', 'xili-tidy-tags' ) . '</li>' .
			'<li>' . __( 'Don’t forget to update by clicking button in left - Actions - titled meta box. ', 'xili-tidy-tags' ) . '</li>' .
			'</ul>';

			$more_infos =
			'<p><strong>' . __( 'For more information:' ) . '</strong></p>' .
			'<p>' . __( '<a href="http://dev.xiligroup.com/xili-tidy-tags" target="_blank">Xili-tidy-tags Plugin Documentation</a>', 'xili-tidy-tags' ) . '</p>' .
			'<p>' . __( '<a href="http://wiki.xiligroup.org/" target="_blank">Xili Wiki Documentation</a>', 'xili-tidy-tags' ) . '</p>' .
			'<p>' . __( '<a href="https://wordpress.org/support/plugin/xili-tidy-tags" target="_blank">Support Forums</a>', 'xili-tidy-tags' ) . '</p>' .
			'<p>' . __( '<a href="http://codex.wordpress.org/" target="_blank">WordPress Documentation</a>', 'xili-tidy-tags' ) . '</p>';

			$screen->add_help_tab(
				array(
					'id'      => 'to-remember',
					'title'   => __( 'Things to remember', 'xili-tidy-tags' ),
					'content' => $to_remember,
				)
			);

			$screen->add_help_tab(
				array(
					'id'      => 'more-infos',
					'title'   => __( 'For more information', 'xili-tidy-tags' ),
					'content' => $more_infos,
				)
			);

		}
		return $contextual_help;
	}

	// called by each pointer
	public function insert_news_pointer( $case_news ) {
			wp_enqueue_style( 'wp-pointer' );
			wp_enqueue_script( 'wp-pointer', false, array( 'jquery' ) );
			++$this->news_id;
			$this->news_case[ $this->news_id ] = $case_news;
	}
	// insert the pointers registered before
	public function print_the_pointers_js() {
		if ( 0 != $this->news_id ) {
			for ( $i = 1; $i <= $this->news_id; $i++ ) {
				$this->print_pointer_js( $i );
			}
		}
	}

	public function print_pointer_js( $indice ) {

		$args = $this->localize_admin_js( $this->news_case[ $indice ], $indice );
		if ( '' != $args['pointerText'] ) {
			// only if user don't read it before
		?>
		<script type="text/javascript">
		//<![CDATA[
		jQuery(document).ready( function() {

		var strings<?php echo $indice; ?> = <?php echo json_encode( $args ); ?>;

	<?php /** Check that pointer support exists AND that text is not empty - inspired www.generalthreat.com */   ?>

	if( 'undefined' != typeof(jQuery().pointer) && '' != strings<?php echo $indice; ?>.pointerText ) {
		jQuery( strings<?php echo $indice; ?>.pointerDiv ).pointer({
			content : strings<?php echo $indice; ?>.pointerText,
			position: { edge: strings<?php echo $indice; ?>.pointerEdge,
				at: strings<?php echo $indice; ?>.pointerAt,
				my: strings<?php echo $indice; ?>.pointerMy
			},
			close : function() {
				jQuery.post( ajaxurl, {
					pointer: strings<?php echo $indice; ?>.pointerDismiss,
					action: 'dismiss-wp-pointer'
				});
			}
		}).pointer('open');
	}
});
		//]]>
		</script>
		<?php
		//offset: strings echo $indice;  .pointerOffset  // deprecated
		}
	}


	/**
	 * News pointer for tabs
	 *
	 * @since 2.6.2
	 *
	 */
	public function localize_admin_js( $case_news, $news_id ) {
		$about = __( 'Docs about xili-tidy-tags', 'xili-tidy-tags' );
		$pointer_edge = '';
		$pointer_at = '';
		$pointer_my = '';
		$pre = ( 'post_tag' == $this->post_tag ) ? '' : '_' . $this->post_tag;

		switch ( $case_news ) {

			case 'xtt_new_version':
				$pointer_text = '<h3>' . esc_js( __( 'xili-tidy-tags updated', 'xili-tidy-tags' ) ) . '</h3>';
				/* translators: */
				$pointer_text .= '<p>' . esc_js( sprintf( __( 'xili-tidy-tags was updated to version %s', 'xili-tidy-tags' ), XILITIDYTAGS_VER ) ) . ' .</p>';
				/* translators: */
				$pointer_text .= '<p>' . esc_js( sprintf( __( 'This version %s is tested with WP 4.3', 'xili-tidy-tags' ), XILITIDYTAGS_VER ) ) . ' .</p>';
				$pointer_text .= '<p>' . esc_js( __( 'Now tags can be grouped using alias feature existing in WP taxonomy and two new template tags for theme are available for files tag.php and taxonomy.php. ', 'xili-tidy-tags' ) ) . ',</p>';

				$pointer_text .= '<p>' . esc_js( __( 'See submenu', 'xili-tidy-tags' ) . ' “<a href="admin.php?page=xili_tidy_tags_settings' . $pre . '">' . __( 'to define groups of tags', 'xili-tidy-tags' ) . "</a>”" ) . ' .</p>';

				$pointer_text .= '<p>' . esc_js( __( 'See submenu', 'xili-tidy-tags' ) . ' “<a href="admin.php?page=xili_tidy_tags_assign' . $pre . '">' . __( 'to assign tags to group', 'xili-tidy-tags' ) . "</a>”" ) . ' .</p>';
				/* translators: */
				$pointer_text .= '<p>' . esc_js( sprintf( esc_html__( 'Before to question dev.xiligroup support, do not forget to visit %s documentation', 'xili-tidy-tags' ), '<a href="http://wiki.xiligroup.org" title="' . $about . '" >wiki</a>' ) ) . ' .</p>';

				$pointer_dismiss = sanitize_key( 'xtt-new-version-' . str_replace( '.', '-', XILITIDYTAGS_VER ) );
				$pointer_div = '#toplevel_page_xili-tidy-tags';
				$pointer_offset = '0 0';
				$pointer_edge = 'left';
				$pointer_my = 'left';
				$pointer_at = 'right';
				break;

			default: // nothing
				$pointer_text = '';
		}

		// inspired from www.generalthreat.com
		// Get the list of dismissed pointers for the user
		$dismissed = explode( ',', (string) get_user_meta( get_current_user_id(), 'dismissed_wp_pointers', true ) );

		if ( in_array( $pointer_dismiss, $dismissed ) && sanitize_key( 'xtt-new-version-' . str_replace( '.', '-', XILITIDYTAGS_VER ) ) == $pointer_dismiss ) {
			$pointer_text = '';
		} elseif ( in_array( $pointer_dismiss, $dismissed ) ) {
			$pointer_text = '';
		}

		return array(
			'pointerText' => html_entity_decode( (string) $pointer_text, ENT_QUOTES, 'UTF-8' ),
			'pointerDismiss' => $pointer_dismiss,
			'pointerDiv' => $pointer_div,
			'pointerEdge' => ( '' == $pointer_edge ) ? 'top' : $pointer_edge,
			'pointerAt' => ( '' == $pointer_at ) ? 'left top' : $pointer_at,
			'pointerMy' => ( '' == $pointer_my ) ? 'left top' : $pointer_my,
			'pointerOffset' => $pointer_offset,
			'newsID' => $news_id,
		);
	}

	/**
	 * private functions for xili-tidy-tags settings
	 *
	 * @since 0.8.0
	 *
	 * fill the content of the boxes (right side and normal)
	 *
	 */
	public function on_sidebox_msg_content( $data = array() ) {
		extract( $data );
		?>
		<h4><?php esc_html_e( 'Note:', 'xili-tidy-tags' ); ?></h4>
		<p><?php echo $message; ?></p>
		<?php
	}

	public function on_sidebox_info_content( $data = array() ) {
		extract( $data);
		$allowed_html = array(
			'a' => array(
				'href' => array(),
				'title' => array(),
			),
			'br' => array(),
			'em' => array(),
			'strong' => array(),
		);
		if ( 'settings' == $xili_tidy_tags_page ) {
			echo '<p style="margin:2px; padding:3px; border:1px solid #ccc;">' . wp_kses( __( 'On this page, the tags groups are defined. The special groups for xili-language plugin are importable.<br /> For debug, some technical infos are displayed in the tables or boxes.<br />', 'xili-tidy-tags' ), $allowed_html ) . '</p>';
		} elseif ( 'assign' == $xili_tidy_tags_page ) {
			echo '<p style="margin:2px; padding:3px; border:1px solid #ccc;">' . wp_kses( __( 'On this page, in a oneshot way, it is possible to assign the tags to one or more groups defined on the other page of <i>xili-tidy-tags</i> plugin. ', 'xili-tidy-tags' ), $allowed_html ) . '</p>';
		}
		?>
		<p><?php echo wp_kses( __( '<strong>xili-tidy-tags</strong> is a tool for grouping tags by language or semantic group. Initially developed to enrich multilingual website powered by xili-language plugin. ', 'xili-tidy-tags' ), $allowed_html ); ?></p>
		<?php
	}

	/*
	 * Admin capabilities setting box
	 * @since 0.9.5
	 *
	 * @updated 1.4.0
	 * Only visible if admin (cap : update_plugins)
	 */
	public function on_sidebox_admin_content( $data = array() ) {
		$editor_set = $this->xili_settings['editor_caps'];
		$selected2 = '';
		$selected3 = '';
		if ( 'caps_setting_grouping' == $editor_set ) {
			$selected3 = ' selected = "selected"';
		} elseif ( 'caps_grouping' == $editor_set ) {
			$selected2 = ' selected = "selected"';
		}
		?>
		<div style="margin:2px; padding:3px; border:1px solid #ccc;">
		<p><?php esc_html_e( 'Here, as admin, set capabilities of the editor:', 'xili-tidy-tags' ); ?></p>
		<select name="editor_caps" id="editor_caps" style="width:80%;">
			<option value="no_caps" ><?php esc_html_e( 'no capability', 'xili-tidy-tags' ); ?></option>
			<option value="caps_grouping" <?php echo $selected2; ?>><?php esc_html_e( 'Grouping', 'xili-tidy-tags' ); ?></option>
			<option value="caps_setting_grouping" <?php echo $selected3; ?>><?php esc_html_e( 'Setting and grouping', 'xili-tidy-tags' ); ?></option>
		</select><br /><br />
		<label for="datatable_js"><?php esc_html_e( 'Disable Datatable javascript', 'xili-tidy-tags' ); ?> : <input id="datatable_js" name="datatable_js" type="checkbox" value="disable" <?php checked( $this->xili_settings['datatable_js'], 'disable' ); ?> /></label>

		<?php
		echo '<p class="submit"><input type="submit" name="editor_caps_submit" value="' . esc_html__( 'Set &raquo;', 'xili-tidy-tags' ) . '" /></p></div>';
	}

	/**
	 * Get_terms without annoying cache
	 *
	 * @since 1.5.0
	 */
	public function no_cache_get_terms( $taxonomy, $args ) {
		global $wpdb;
		$defaults = array(
			'orderby' => 'name',
			'order' => 'ASC',
			'hide_empty' => true,
			'exclude' => array(),
			'exclude_tree' => array(),
			'include' => array(),
			'number' => '',
			'fields' => 'all',
			'slug' => '',
			'parent' => '',
			'hierarchical' => true,
			'child_of' => 0,
			'get' => '',
			'name__like' => '',
			'pad_counts' => false,
			'offset' => '',
			'search' => '',
		);
		$args = wp_parse_args( $args, $defaults );
		extract( $args, EXTR_SKIP );
		$orderby = 't.name';
		if ( ! empty( $orderby ) ) {
			$orderby = "ORDER BY $orderby";
		} else {
			$order = '';
		}
		$where = '';
		if ( '' !== $parent ) {
			$parent = (int) $parent;
			$where .= " AND tt.parent = '$parent'";
		}

		$selects = array();
		switch ( $fields ) {
			case 'all':
				$selects = array( 't.*', 'tt.*' );
				break;
			case 'ids':
			case 'id=>parent':
				$selects = array( 't.term_id', 'tt.parent', 'tt.count' );
				break;
			case 'names':
				$selects = array( 't.term_id', 'tt.parent', 'tt.count', 't.name' );
				break;
			case 'count':
				$orderby = '';
				$order = '';
				$selects = array( 'COUNT(*)' );
		}
		$select_this = implode( ', ', apply_filters( 'get_terms_fields', $selects, $args ) );
		$query = "SELECT $select_this FROM $wpdb->terms AS t INNER JOIN $wpdb->term_taxonomy AS tt ON t.term_id = tt.term_id WHERE tt.taxonomy IN ( %s ) $where $orderby $order";

		// $taxonomy set as param (wp 3.9 - placeholder notice )
		$terms = $wpdb->get_results( $wpdb->prepare( $query, $taxonomy ) );

		return $terms;
	}

	/*
	 * Action's box
	 */
	public function on_sidebox_import_content( $data = array() ) {
		extract( $data );
		if ( 'post_tag' == $this->post_tag ) { // 1.5.5
			echo '<div style="margin:2px; padding:3px; border:1px solid #ccc;">';
			echo '<p>' . esc_html__( 'Add a tag\'s group for a chosen category', 'xili-tidy-tags' ) . '</p>';
			/* build the selector of available categories */
			$categories = get_categories( array( 'get' => 'all' ) ); /* even if cat is empty */
			echo '<select name="catsforgroup" id="catsforgroup" style="width:100%;">';
			echo '<option value="no" >' . esc_html__( 'choose a category', 'xili-tidy-tags' ) . '</option>';
			foreach ( $categories as $cat ) {
				$catinside = term_exists( $cat->slug, $this->tidy_taxonomy );
				if ( 0 == $catinside && 1 != $cat->term_id ) {
					echo '<option value="' . $cat->term_id . '" >' . $cat->name . '</option>';
				}
			}
			echo '</select>';
			echo '<p>' . esc_html__( 'Choose a parent tag\'s group', 'xili-tidy-tags' ) . '</p>';
			$listterms = get_terms( $this->tidy_taxonomy, array( 'hide_empty' => false ) );
			?>
			<select name="tags_parent" id="tags_parent" style="width:100%;">
				<option value="no_parent" ><?php esc_html_e( 'no parent', 'xili-tidy-tags' ); ?></option>
			<?php

			foreach ( $listterms as $curterm ) {
				if ( 0 == $curterm->parent && $curterm->term_id != $this->langgroupid ) {
					echo '<option value="' . $curterm->term_id . '" >' . $curterm->name . '</option>';
				}
			}
			?>
			</select>
			<br />
			<?php
			echo '<p class="submit"><input type="submit" name="importacat" value="' . esc_html__( 'Add &raquo;', 'xili-tidy-tags' ) . '" /></p>';
			echo '<p>' . esc_html__( 'See docs to set xili_tidy_tag_cloud function or widget to implement in theme…', 'xili-tidy-tags' ) . '</p>';
			echo '</div>';
		}
		echo '<div style="margin:2px; padding:3px; border:1px solid #ccc;">';
		if ( ! class_exists( 'xili_language' ) ) {
			?>
			<p class="submit"><?php esc_html_e( 'xili-language plugin is not activated. ', 'xili-tidy-tags' ); ?> </p>
			<?php
		} else {

			$res = term_exists( $this->xili_settings['tidylangsgroupname'], $this->tidy_taxonomy );
			if ( is_array( $res ) ) {
				$childterms = $this->no_cache_get_terms(
					$this->tidy_taxonomy,
					array(
						'hide_empty' => false,
						'fields' => 'all',
						'parent' => $res['term_id'],
					)
				);
			}
			//echo 'nb' .count( $childterms);
			if ( $res && ! empty( $childterms ) ) {
				?>
				<p><?php esc_html_e( 'The group of languages is set for use with xili-language plugin. ', 'xili-tidy-tags' ); ?> </p>
				<?php

				if ( current_user_can( 'xili_tidy_admin_set' ) ) { //current_user_can( 'xili_tidy_admin_set' )
					$langinxtt = array();
					foreach ( $childterms as $childterm ) {
						$langinxtt[] = $childterm->name;
					}

					$langinxl = array();
					$listlanguages = get_terms( TAXONAME, array( 'hide_empty' => false ) );
					foreach ( $listlanguages as $language ) {
						$langinxl[] = $language->name;
					}

					if ( array_diff( $langinxl, $langinxtt ) != array() ) { // since 1.5.0
						$s = implode( ', ', array_diff( $langinxl, $langinxtt ) );
					?>
						<p><?php /* translators: */ printf( esc_html__( 'The group of languages in xili-language plugin has been changed. ', 'xili-tidy-tags' ) . ' ( %s )', $s ); ?> </p>
					<?php
						echo '<p class="submit">' . esc_html__( 'It is possible to update the group of languages. ', 'xili-tidy-tags' ) . '</p>';
						echo '<p class="submit"><input type="submit" name="importxililanguages" value="' . esc_html__( 'Update &raquo;', 'xili-tidy-tags' ) . '" /></p>';
					}
					if ( array() != array_diff( $langinxtt, $langinxl ) ) {
						// ( count( $langinxtt) != count( $langinxl) )
						$s = implode( ', ', array_diff( $langinxtt, $langinxl ) );
						/* translators: */
						echo '<p class="submit">' . sprintf( esc_html__( 'One or more language(s) here are not present in active xili-language list. ', 'xili-tidy-tags' ) . ' ( %s )', $s ) . '</p>';
					}
				}
			} else { /* since 0.9.5 */
				if ( current_user_can( 'xili_tidy_admin_set' ) ) {
					$count_xl = wp_count_terms( TAXONAME ); /* count a minima one language */
					if ( $count_xl > 0 ) {
						echo '<p class="submit">' . esc_html__( 'It is possible to import the group of languages. ', 'xili-tidy-tags' ) . '</p>';
						echo '<p class="submit"><input type="submit" name="importxililanguages" value="' . esc_html__( 'Import…', 'xili-tidy-tags' ) . '" /></p>';

					} else {
						echo '<p class="submit">' . esc_html__( 'Go to settings of xili-language plugin and add languages', 'xili-tidy-tags' ) . '</p>';
					}
				} else {
					echo '<p class="submit">' . esc_html__( 'See administrator for language settings. ', 'xili-tidy-tags' ) . '</p>';
				}
			}
		}
		echo '</div>';

	}

	public function on_normal_group_tags_list_content( $data = array() ) {
		extract( $data );
		?>

					<table class="widefat" style="clear:none;">
						<thead>
						<tr>
						<th scope="col" style="text-align: center"><?php esc_html_e( 'ID', 'xili-tidy-tags' ); ?></th>
						<th scope="col"><?php esc_html_e( 'Name', 'xili-tidy-tags' ); ?></th>
						<th scope="col"><?php esc_html_e( 'Description', 'xili-tidy-tags' ); ?></th>
						<th scope="col"><?php esc_html_e( 'Group slug', 'xili-tidy-tags' ); ?></th>
						<th scope="col"><?php esc_html_e( 'Group taxo ID', 'xili-tidy-tags' ); ?></th>
						<th scope="col"><?php esc_html_e( 'Parent', 'xili-tidy-tags' ); ?></th>
						<th scope="col" width="90" style="text-align: center"><?php esc_html_e( 'Tags' ); ?></th>
						<th colspan="2" style="text-align: center"><?php esc_html_e( 'Action' ); ?></th>
						</tr>
						</thead>
						<tbody id="the-list">
							<?php $this->xili_tags_group_row(); /* the lines */ ?>
						</tbody>
					</table>

		<?php
	}

	public function on_normal_group_form_content( $data = array() ) {
		extract( $data ); /* form to add or edit group */
		?>

		<h2 id="addgroup" <?php if ( $action == 'delete' ) echo 'style="color:#FF1111;"'; ?>><?php echo $formtitle; ?></h2>
		<?php if ( 'edit' == $action || 'delete' == $action ) :?>
			<input type="hidden" name="tagsgroup_term_id" value="<?php echo $tagsgroup->term_id; ?>" />
			<input type="hidden" name="tagsgroup_parent" value="<?php echo $tagsgroup->parent; ?>" />
		<?php
		endif;
		$disabled = ( 'delete' == $action ) ? ' disabled="disabled"' : '';
		?>
		<table class="editform" width="100%" cellspacing="2" cellpadding="5">
			<tr>
				<th width="33%" scope="row" valign="top" align="right"><label for="tagsgroup_name"><?php esc_html_e( 'Name', 'xili-tidy-tags' ); ?></label>:&nbsp;</th>
				<td width="67%"><input name="tagsgroup_name" id="tagsgroup_name" type="text" value="<?php if ( isset( $tagsgroup) ) echo esc_attr( $tagsgroup->name ); ?>" size="40" <?php echo $disabled; ?> /></td>
			</tr>
			<tr>
				<th scope="row" valign="top" align="right"><label for="tagsgroup_nicename"><?php esc_html_e( 'tags group slug', 'xili-tidy-tags' ); ?></label>:&nbsp;</th>
				<td><input name="tagsgroup_nicename" id="tagsgroup_nicename" type="text" value="<?php
					if ( isset( $tagsgroup ) ) {
						echo esc_attr( $tagsgroup->slug );
					}
					?>" size="40"<?php echo $disabled; ?> /></td>
			</tr>
			<tr>
				<th scope="row" valign="top" align="right"><label for="tagsgroup_description"><?php esc_html_e( 'Description', 'xili-tidy-tags' ); ?></label>:&nbsp;</th>
				<td><input name="tagsgroup_description" id="tagsgroup_description" size="40" value="<?php if ( isset( $tagsgroup)) echo esc_attr( $tagsgroup->description ); ?>" <?php echo $disabled; ?> /></td>

			</tr>
			<tr class="form-field">
				<th scope="row" valign="top" align="right"><label for="tagsgroup_parent"><?php esc_html_e( 'kinship', 'xili-tidy-tags' ); ?></label> :&nbsp;</th>
				<td>
			<?php
			if ( isset( $tagsgroup ) ) {
				$this->xili_selectparent_row( $tagsgroup->term_id, $tagsgroup, $action );
				/* choice of parent line*/
			} else {
				$this->xili_selectparent_row( 0, $tagsgroup, $action );
			}
			?>
				</td>
			</tr>
			<tr>
			<th><p class="submit"><input type="submit" name="reset" value="<?php echo $cancel_text; ?>" /></p></th>
			<td>
			<p class="submit"><input type="submit" name="submit" value="<?php echo $submit_text; ?>" /></p>
			</td>
			</tr>
		</table>
	<?php
	}

	public function xili_selectparent_row( $term_id = 0, $tagsgroup, $action ) {
		if ( 0 == $term_id ) {
				$listterms = get_terms( $this->tidy_taxonomy, array( 'hide_empty' => false ) );
				?>
				<select name="tagsgroup_parent" id="tagsgroup_parent" style="width:100%;">
					<option value="no_parent" ><?php esc_html_e( 'no parent', 'xili-tidy-tags' ); ?></option>
				<?php
				foreach ( $listterms as $curterm ) {
					if ( 0 == $curterm->parent ) {
						if ( current_user_can( 'xili_tidy_admin_set' ) ) {
							$possible = true;
						} elseif ( $curterm->term_id == $this->langgroupid ) {
							$possible = false;
						} else {
							$possible = true;
						}
						if ( $possible ) {
							echo '<option value="' . $curterm->term_id . '" >' . $curterm->name . '</option>';
						}
					}
				}
				?>
				</select>
				<br />
				<?php
				esc_html_e( 'Select the parent if necessary', 'xili-tidy-tags' );
		} else {
			if ( 0 == $tagsgroup->parent ) {
				$listterms = get_terms(
					$this->tidy_taxonomy,
					array(
						'hide_empty' => false,
						'parent' => $term_id,
					)
				);
					// display childs
				if ( ! empty( $listterms ) ) {
					esc_html_e( 'parent of: ', 'xili-tidy-tags' );
					echo '<ul>';
					foreach ( $listterms as $curterm ) {
						echo '<li value="' . $curterm->term_id . '" >' . $curterm->name . '</li>';
					}
					echo '</ul>';
				} else {
					esc_html_e( 'no child now', 'xili-tidy-tags' ) . '<br /><br />';
				}
				/* if modify*/
				if ( 'edit' == $action ) {
					$listterms = get_terms( $this->tidy_taxonomy, array( 'hide_empty' => false ) );
					?>
			<select name="tagsgroup_parent" id="tagsgroup_parent" style="width:100%;">
						<option value="no_parent" ><?php esc_html_e( 'no parent', 'xili-tidy-tags' ); ?></option>
					<?php
					foreach ( $listterms as $curterm ) {
						if ( 0 == $curterm->parent && $curterm->term_id != $term_id ) {
							echo '<option value="' . $curterm->term_id . '" >' . $curterm->name . '</option>';
						}
					}
				?>
			</select>
			<br />
			<?php
					esc_html_e( 'Select the parent if necessary', 'xili-tidy-tags' );
				}
			} else {
				/* if modify*/
				$parent_term = get_term( (int) $tagsgroup->parent, $this->tidy_taxonomy, OBJECT, 'edit' );
				if ( 'delete' == $action ) {
					esc_html_e( 'child of: ', 'xili-tidy-tags' );
					echo $parent_term->name;
					?>
						<input type="hidden" name="tagsgroup_parent" value="<?php echo $parent_term->term_id; ?>" />
					<?php
				} else {
					$listterms = get_terms( $this->tidy_taxonomy, array( 'hide_empty' => false ) );
					?>
					<select name="tagsgroup_parent" id="tagsgroup_parent" style="width:100%;">
						<option value="no_parent" ><?php esc_html_e( 'no parent', 'xili-tidy-tags' ); ?></option>
					<?php
					foreach ( $listterms as $curterm ) {
						if ( 0 == $curterm->parent && $curterm->term_id != $term_id ) {
							//$checked = ( $parent_term->term_id == $curterm->term_id) ? 'selected="selected"' :'';
							echo '<option value="' . $curterm->term_id . '" ' . selected( $parent_term->term_id, $curterm->term_id, false ) . ' >' . $curterm->name . '</option>';
						}
					}
					?>
					</select>
			<br />
			<?php
				esc_html_e( 'Modify the parent if necessary', 'xili-tidy-tags' );
				}
			}
		}
	}



	/*
	 * Import or update the terms (languages) set by xili-language	 *
	 * @updated 1.5.0
	 */
	public function xili_langs_import_terms() {

		$term = $this->xili_settings['tidylangsgroupname'];
		$error = false;
		$args = array(
			'alias_of' => '',
			'description' => 'default lang group',
			'parent' => 0,
			'slug' => $this->xili_settings['tidylangsgroup'],
		);
		$theids = term_exists( $term, $this->tidy_taxonomy ); // impossible to use get_term as formerly !!!
		if ( ! $theids ) {
			$theids = wp_insert_term( $term, $this->tidy_taxonomy, $args );
			if ( ! is_wp_error( $theids ) ) {
				$this->langgroupid = $theids['term_id'];
			} else {
				$error = true;
			}
		}
		if ( $error ) {
			return false;
		} else {
			$listlanguages = get_terms( TAXONAME, array( 'hide_empty' => false ) );
			foreach ( $listlanguages as $language ) {
				$args = array(
					'alias_of' => '',
					'description' => $language->description,
					'parent' => $this->langgroupid,
					'slug' => $language->slug . '-' . $this->xili_settings['tidylangsgroup'],
				); // slug to be compatible with former release of WP

				$ifhere = term_exists( $language->name, $this->tidy_taxonomy );
				if ( ! $ifhere ) {
					$res = wp_insert_term( $language->name, $this->tidy_taxonomy, $args );
				} else {
					$error = true;
				}
			}
			return ! $error;
		}
	}

	/*
	 * Display the rows of group of tags
	 *
	 * @updated since 1.3.0 - use now walker class to sort Tag's groups
	 */
	public function xili_tags_group_row() {
		$listtagsgroups = get_terms(
			$this->tidy_taxonomy,
			array(
				'hide_empty' => false,
				'get' => 'all',
			)
		);

		if ( is_wp_error( $listtagsgroups ) || array() == $listtagsgroups ) {
			/* import */
			if ( class_exists( 'xili_language' ) ) { /* xili-language is present */
				$res = $this->xili_langs_import_terms();

			} else {
				/*create a default line with the default group*/
				$term = 'tidy group';
				$args = array(
					'alias_of' => '',
					'description' => 'default xili tidy tags group',
					'parent' => 0,
				);
				$resgroup = wp_insert_term( $term, $this->tidy_taxonomy, $args );
				$res = ! is_wp_error( $resgroup );
			}
			if ( $res ) {
				$listtagsgroups = $this->no_cache_get_terms(
					$this->tidy_taxonomy,
					array(
						'hide_empty' => false,
						'get' => 'all',
					)
				);
			}
		} else {

			$listtagsgroupssorted = $this->create_taggrouplist_sorted( walk_TagGroupList_sorted( $listtagsgroups, 3, null, null ) );

			$class = '';
			if ( $listtagsgroupssorted ) { // @since 1.4.2 - no warning if no groups at init
				foreach ( $listtagsgroupssorted as $tagsgroup ) {
					$class = ( ( defined( 'DOING_AJAX' ) && DOING_AJAX ) || " class='alternate'" == $class ) ? '' : " class='alternate'";

					$tagsgroup->count = number_format_i18n( $tagsgroup->count );
					$posts_count = ( $tagsgroup->count > 0 ) ? "<a href='edit.php?lang=$tagsgroup->term_id'>$tagsgroup->count</a>" : $tagsgroup->count;
					/* since 0.9.5 */
					if ( current_user_can( 'xili_tidy_editor_set' ) ) { /* all admin only */
						$possible = true;
					} elseif ( current_user_can( 'xili_tidy_editor_set' ) ) { /* editor if set */
						if ( $tagsgroup->term_id == $this->langgroupid || $tagsgroup->parent == $this->langgroupid ) {
							$possible = false;
						} else {
							$possible = true;
						}
					} else {
						$possible = false;
					}
					$pre = ( 'post_tag' == $this->post_tag ) ? 'xili_tidy_tags_settings' : 'xili_tidy_tags_settings_' . $this->post_tag;
					if ( true === $possible ) {
						$edit = "<a href='?page=" . $pre . '&amp;action=edit&amp;term_id=' . $tagsgroup->term_id . "' >" . __( 'Edit' ) . '</a></td>';
						/* delete link &amp;action=edit&amp;term_id=' . $tagsgroup->term_id. "*/
						$edit .= "<td><a href='?page=" . $pre . '&amp;action=delete&amp;term_id=' . $tagsgroup->term_id . "' class='delete'>" . __( 'Delete' ) . '</a>';
					} else {
						$edit = __( 'no capability', 'xili-tidy-tags' ) . '</td><td>';
					}

					$line = "<tr id='cat-$tagsgroup->term_id'$class>
					<th scope='row' style='text-align: center'>$tagsgroup->term_id</th>
					<td> ";
					$tabb = ( 0 != $tagsgroup->parent ) ? ' –' : '';
					$tagsgroupname = ( 0 == $tagsgroup->parent ) ? '<strong>' . $tagsgroup->name . '</strong>' : $tagsgroup->name;
					$line .= "$tabb $tagsgroupname</td>
					<td>$tagsgroup->description</td>
					<td>$tagsgroup->slug</td>
					<td>$tagsgroup->term_taxonomy_id</td>
					<td>$tagsgroup->parent</td>
					<td align='center'>$tagsgroup->count</td>
					<td>$edit</td>\n\t</tr>\n"; /*to complete*/

					echo $line;
				}
			}
		}
	}

	/**
	 * @updated 1.5.0 with datatables js (ex widefat)
	 *
	 */
	public function on_sub_normal_tags_list_content( $data = array() ) {
		extract( $data );
		?>

		<div id="topbanner">
		</div>
		<?php if ( '' == $this->xili_settings['datatable_js'] ) { ?>
		<div id="tableupdating" ><br /><br /><h2><?php esc_html_e( 'Creating table of tags !', 'xili-tidy-tags' ); ?></h2>
		</div>
		<table class="display" id="assigntable" style="visibility:hidden; clear:none; font-size:10px; line-height:1.8em;" >
		<?php } else { ?>
		<table class="display" id="assigntable" style="visibility:visible; clear:none;">
		<?php } ?>

						<thead>
						<tr>
							<th scope="col" class="center colid" ><?php esc_html_e( 'ID', 'xili-tidy-tags' ); ?></th>
							<th scope="col" class="colname" ><?php esc_html_e( 'Name', 'xili-tidy-tags' ); ?></th>
							<th scope="col" class="center colposts"><?php esc_html_e( 'Posts' ); ?></th>
							<th colspan="2" class="colgrouping"><?php esc_html_e( 'Group(s) to choose', 'xili-tidy-tags' ); ?></th>
						</tr>
						</thead>
						<tbody id="the-list">
							<?php $this->xili_tags_row( $tagsnamelike, $tagsnamesearch ); ?>
						</tbody>
						<tfoot>
							<tr>
								<th><?php esc_html_e( 'ID', 'xili-tidy-tags' ); ?></th>
								<th><?php esc_html_e( 'Name', 'xili-tidy-tags' ); ?></th>
								<th><?php esc_html_e( 'Posts' ); ?></th>
								<th><?php esc_html_e( 'Group(s) to choose', 'xili-tidy-tags' ); ?></th>
							</tr>
						</tfoot>
					</table>
		<div id="bottombanner">
		</div>


		<?php

	}

	/**
	 *
	 * @updated 1.5.3
	 */
	public function on_sub_sidebox_action_content( $data = array() ) {
		extract( $data );
		?>
		<p><?php esc_html_e( 'After checking or unchecking do not forget to click update button !', 'xili-tidy-tags' ); ?></p>
		<p class="submit"><input type="submit" class="button-primary" id="update" name="update" value="<?php echo $submit_text; ?>" />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<input type="submit" name="reset" value="<?php echo $cancel_text; ?>" /></p>

		<fieldset style="margin:2px; padding:3px; border:1px solid #ccc;"><legend><?php esc_html_e( 'Sub list of tags', 'xili-tidy-tags' ); ?></legend>

			<label for="tagsnamelike"><?php esc_html_e( 'Starting with:', 'xili-tidy-tags' ); ?></label>
			<input name="tagsnamelike" id="tagsnamelike" type="text" value="<?php echo esc_attr( $tagsnamelike ); ?>" /><br />

			<label for="tagsnamesearch"><?php esc_html_e( 'Containing:', 'xili-tidy-tags' ); ?></label>
			<input name="tagsnamesearch" id="tagsnamesearch" type="text" value="<?php echo esc_attr( $tagsnamesearch ); ?>" /><br /><br />
			<label for="tagsfromgroup"><?php esc_html_e( 'Choose:', 'xili-tidy-tags' ); ?></label>
		<?php
		$listtagsgroups = get_terms( $this->tidy_taxonomy, array( 'hide_empty' => false ) );

		if ( ! is_wp_error( $listtagsgroups ) && array() != $listtagsgroups ) {
			$listtagsgroupssorted = $this->create_taggrouplist_sorted( walk_taggrouplist_sorted( $listtagsgroups, 3, null, null ) );
			//$listtagsgroupssorted = $listtagsgroups;
			?>
				<select name="tagsgroup_from_select" id="tagsgroup_from_select" style="width:45%;">
					<option value="no_select" ><?php esc_html_e( 'every', 'xili-tidy-tags' ); ?></option>
					<option value="onlyuncheckedtags" <?php selected( $this->onlyuncheckedtags ); ?>><?php esc_html_e( 'Unchecked only', 'xili-tidy-tags' ); ?></option>
			<?php
			$show = false;

			foreach ( $listtagsgroupssorted as $curterm ) {
				$ttab = ( 0 == $curterm->parent ) ? '' : '– ';
				if ( $this->fromgroupselect == $curterm->term_id ) {
					$selected = 'selected="selected"';
					if ( 0 == $curterm->parent ) {
						$listlanggroups = get_terms(
							$this->tidy_taxonomy,
							array(
								'hide_empty' => true,
								'parent' => $curterm->term_id,
							)
						);
						if ( ! is_wp_error( $listlanggroups ) && $listlanggroups ) {
							$show = true;
						}
					}
				} else {
					$selected = '';
				}
				echo '<option value="' . $curterm->term_id . '" ' . $selected . ' >' . $ttab. $curterm->name . '</option>';

			}

			?>
				</select>
		<?php
		}
		if ( false == $this->onlyuncheckedtags && 0 != $this->fromgroupselect ) {
			?>
			<br /><label for="xili_group_not_select"><?php esc_html_e( 'Exclude this group', 'xili-tidy-tags' ); ?> <input id="xili_group_not_select" name="xili_group_not_select" type="checkbox" value="not" <?php checked( $this->groupexclude, true ); ?> /></label>
			<?php if ( $show ) { ?>
			&nbsp;–&nbsp;<label for="xili_group_only_parent"><?php esc_html_e( 'No childs', 'xili-tidy-tags' ); ?> <input id="xili_group_only_parent" name="xili_group_only_parent" type="checkbox" value="onlyparent" <?php checked( $this->onlyparent, true); ?> /></label>
			<?php } ?>
			<br />
			<?php
			if ( true == $this->groupexclude ) {
				?>
				<label for="xili_uncheckedtags"><?php esc_html_e( 'Include unchecked', 'xili-tidy-tags' ); ?> <input id="xili_uncheckedtags" name="xili_uncheckedtags" type="checkbox" value="include" <?php checked( $this->uncheckedtags, true ); ?> /></label>
				<?php
			}
		}
		?>
			<p class="submit"><input type="submit" id="tagssublist" name="tagssublist" value="<?php esc_html_e( 'Sub select…', 'xili-tidy-tags' ); ?>" />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<input type="submit" id="notagssublist" name="notagssublist" value="<?php esc_html_e( 'No select…', 'xili-tidy-tags' ); ?>" /></p>
		</fieldset>
		<?php /* only show one group to select */ ?>
		<fieldset style="margin:2px; padding:3px; border:1px solid #ccc;"><legend><?php esc_html_e( 'Columns: Group selection', 'xili-tidy-tags' ); ?></legend>
		<?php
		if ( ! is_wp_error( $listtagsgroups ) && array() != $listtagsgroups ) {
			?>
			<select name="tagsgroup_parent_select" id="tagsgroup_parent_select" style="width:100%;">
				<option value="no_select" ><?php esc_html_e( 'No sub-selection', 'xili-tidy-tags' ); ?></option>
			<?php
			foreach ( $listtagsgroups as $curterm ) {
				if ( 0 == $curterm->parent ) {
					//$checked = ( $this->subselect == $curterm->term_id) ? 'selected="selected"' :'';
					echo '<option value="' . $curterm->term_id . '" ' . selected( $this->subselect, $curterm->term_id, false ) . ' >' . $curterm->name . '</option>';
				}
			}
			?>
			</select>
			<?php
		}
		?>
		<br /> <p class="submit"><input type="submit" id="subselection" name="subselection" value="<?php esc_html_e( 'Sub select…', 'xili-tidy-tags' ); ?>" /></p></fieldset>
		<?php
	}
	/**
	 * The rows of the tags and checkboxes to assign group(s)
	 *
	 * @since 0.8.0
	 * @updated 1.3.0 - Call walker instantiation
	 * @uses
	 * @param
	 * @return the rows for admin ui
	 */
	public function xili_tags_row( $tagsnamelike = '', $tagsnamesearch = '' ) {
		$listgroups = get_terms(
			$this->tidy_taxonomy,
			array(
				'hide_empty' => false,
				'get' => 'all',
			)
		);

		if ( ! is_wp_error( $listgroups ) && array() != $listgroups ) {
			$hiddenline = array();
			$edit = '';
			$i = 0;
			$listgroupids = array();
			$sub_listgroups = array();
			$subselectgroups = array();
			if ( $this->subselect > 0 ) {
				$subselectgroups[] = $this->subselect; /* the parent group and */
				/*childs of */
				$listterms = get_terms(
					$this->tidy_taxonomy,
					array(
						'hide_empty' => false,
						'parent' => $this->subselect,
					)
				);
				if ( ! empty( $listterms ) ) {
					foreach ( $listterms as $curterm ) {
						$subselectgroups[] = $curterm->term_id;
					}
				}
			}

			if ( ! empty( $subselectgroups ) ) {
				/* columns sub-selection */
				foreach ( $listgroups as $group ) {
					$listgroupids[] = $group->term_id;
					if ( in_array( $group->term_id, $subselectgroups ) ) {
						$sub_listgroups[] = $group;
					} else {
						 $hiddenline[] = $group->term_id; /* keep line values */
					}
				}
				$editformat = walk_TagGroupList_tree_row( $sub_listgroups, 3, null );
			} else {
				foreach ( $listgroups as $group ) {
					$listgroupids[] = $group->term_id;
				}
				$editformat = walk_TagGroupList_tree_row( $listgroups, 3, null );
			}

			if ( 0 == $this->fromgroupselect && false === $this->onlyuncheckedtags ) {
				$listtags = get_terms(
					$this->post_tag,
					array(
						'hide_empty' => false,
						'get' => 'all',
						'name__like' => $tagsnamelike,
						'search' => $tagsnamesearch,
					)
				);
			} else { /* since 1.3.0 */
				if ( false === $this->onlyuncheckedtags ) {
					// one group
					$group_id[] = $this->fromgroupselect;
					if ( false === $this->onlyparent ) {
						$childterms = get_terms(
							$this->tidy_taxonomy,
							array(
								'hide_empty' => false,
								'parent' => $this->fromgroupselect,
							)
						);
						if ( ! empty( $childterms ) ) {
							foreach ( $childterms as $childterm ) { /* if group is a parent, add all childs */
								$group_id[] = $childterm->term_id;
							}
						}
					}
				} else {
					// only unchecked
					$listgroups = get_terms(
						$this->tidy_taxonomy,
						array(
							'hide_empty' => false,
							'get' => 'all',
						)
					);
					foreach ( $listgroups as $group ) {
						$group_id[] = $group->term_id;
					}
					$this->groupexclude = true;
					$this->uncheckedtags = true;
				}
				$listtags = xtt_get_terms_of_groups_new(
					$group_id,
					$this->tidy_taxonomy,
					$this->post_tag,
					array(
						'hide_empty' => false,
						'get' => 'all',
						'name__like' => $tagsnamelike,
						'search' => $tagsnamesearch,
						'orderby' => 'name',
					),
					$this->groupexclude,
					$this->uncheckedtags
				);
			}

			$class = '';
			if ( ! is_wp_error( $listtags ) ) {
				foreach ( $listtags as $tag ) {
					$class = ( ( defined( 'DOING_AJAX' ) && DOING_AJAX ) || " class='alternate'" == $class ) ? '' : " class='alternate'";
					$tag->count = number_format_i18n( $tag->count );

					if ( 'post_tag' == $this->post_tag ) {
						$posts_count = ( $tag->count > 0 ) ? "<a href='edit.php?tag=$tag->name'>$tag->count</a>" : $tag->count;
					} else {
						$posts_count = ( $tag->count > 0 ) ? "<a href='edit.php?" . $this->post_tag . '=' . $tag->name . '&amp;post_type=' . $this->post_tag_post_type . '>' . $tag->count . '</a>' : $tag->count;
					}
					$edit = sprintf( $editformat, $tag->term_id );
					$hiddenlines = '';
					foreach ( $listgroupids as $groupid ) {
						if ( is_object_in_term( $tag->term_id, $this->tidy_taxonomy, (int) $groupid ) ) {
							$edit = str_replace( '"checked' . $groupid . '"', 'checked="checked"', $edit ); // 1.7
							if ( in_array( $groupid, $hiddenline ) ) {
								$hiddenlines .= '<input type="hidden" name="line-' . $tag->term_id . '-' . $groupid . '" value="' . $tag->term_id . '" />';
							}
						} else {
							$edit = str_replace( '"checked' . $groupid . '"', '', $edit ); // 1.7
						}
					}
					//
					if ( 'post_tag' == $this->post_tag ) {
						$what = '&amp;tag_ID=' . $tag->term_id;
					} else {
						$what = '&amp;taxonomy=' . $this->post_tag . '&amp;tag_ID=' . $tag->term_id . '&amp;post_type=' . $this->post_tag_post_type;
					}
					$line = "<tr id='cat-$tag->term_id'$class>
					<td class='termid' id='termid-{$tag->term_id}' scope='row' style='text-align: center'>$tag->term_id</td>
					<td> <a href='edit-tags.php?action=edit" . $what . "'>" . $tag->name . "</a> </td>
					<td align='center'>$posts_count</td>
					<td>$edit\n$hiddenlines</td>\n\t</tr>\n"; /*to complete*/
					echo $line;
				}
			} else {
				echo '<tr><td>ERROR</td><td> in TAXONOMY: ' . $this->tidy_taxonomy . '</td><td>ERROR</td><td>ERROR</td></tr>';
				if ( ! taxonomy_exists( $this->post_tag ) ) {
					echo '<tr><td>ERROR</td><td> TAXONOMY: ' . $this->post_tag . ' NOT SET</td><td>ERROR</td><td>ERROR</td></tr>'; // 1.9 fixes
				}
			}
		} else {
			echo '<tr><td>ERROR</td><td> in TAXONOMY: ' . $this->tidy_taxonomy . '</td><td>ERROR</td><td>ERROR</td></tr>';
		}
	}




	public function check_other_xili_plugins() {
		$list = array();
		//if ( class_exists( 'xili_language' ) ) $list[] = 'xili-language';
		if ( class_exists( 'xili_language' ) ) {
			$list[] = 'xili-language';
		}
		if ( class_exists( 'xili_dictionary' ) ) {
			$list[] = 'xili-dictionary';
		}
		if ( class_exists( 'xilithemeselector' ) ) {
			$list[] = 'xilitheme-select';
		}
		if ( function_exists( 'insert_a_floom' ) ) {
			$list[] = 'xili-floom-slideshow';
		}
		if ( class_exists( 'xili_postinpost' ) ) {
			$list[] = 'xili-postinpost';
		}
		return implode( ', ', $list );
	}

	public function on_sidebox_mail_content( $data ) {
		extract( $data );
		global $wp_version;
		$theme = ( isset( $this->xili_settings['theme'] ) ) ? $this->xili_settings['theme'] : '';
		$wplang = ( isset( $this->xili_settings['wplang'] ) ) ? $this->xili_settings['wplang'] : '';
		$xiliplug = ( isset( $this->xili_settings['xiliplug'] ) ) ? $this->xili_settings['xiliplug'] : '';
		if ( '' != $emessage ) {
			?>
			<h4><?php esc_html_e( 'Note:', 'xili-tidy-tags' ); ?></h4>
			<p><strong><?php echo $emessage; ?></strong></p>
		<?php
		}
		?>
		<fieldset style="margin:2px; padding:12px 6px; border:1px solid #ccc;"><legend><?php echo esc_html_e( 'Mail to dev.xiligroup', 'xili-tidy-tags' ); ?></legend>
		<label for="ccmail"><?php esc_html_e( 'Cc: (Reply to:)', 'xili-tidy-tags' ); ?>
		<input class="widefat" id="ccmail" name="ccmail" type="text" value="<?php bloginfo( 'admin_email' ); ?>" /></label><br /><br />
		<?php
		if ( false === strpos( get_bloginfo( 'url' ), 'local' ) ) {
				?>
			<label for="urlenable">
				<input type="checkbox" id="urlenable" name="urlenable" value="enable" <?php checked( ( isset( $this->xili_settings['url'] ) && $this->xili_settings['url']=='enable' ), true, true ); ?> />&nbsp;<?php bloginfo( 'url' ); ?>
			</label><br />
			<?php
		} else {
			?>
			<input type="hidden" name="onlocalhost" id="onlocalhost" value="localhost" />
			<?php
		}
			?>
		<br /><em><?php esc_html_e( 'When checking and giving detailled infos, support will be better !', 'xili-tidy-tags' ); ?></em><br />
		<label for="themeenable">
			<input type="checkbox" id="themeenable" name="themeenable" value="enable" <?php checked( $theme, 'enable', true ); ?> />&nbsp;<?php echo 'Theme name= ' . get_option( 'stylesheet' ); ?>
		</label><br />
		<?php if ( '' != $this->get_wplang() ) { ?>
		<label for="wplangenable">
			<input type="checkbox" id="wplangenable" name="wplangenable" value="enable" <?php checked( $wplang, 'enable', true ); ?> />&nbsp;<?php echo 'WPLANG= ' . $this->get_wplang(); ?>
		</label><br />
		<?php } ?>
		<label for="versionenable">
			<input type="checkbox" id="versionenable" name="versionenable" value="enable" <?php if ( isset( $this->xili_settings['version-wp'] ) ) checked( $this->xili_settings['version-wp'],'enable', true); ?> />&nbsp;<?php echo 'WP version: ' . $wp_version; ?>
		</label><br /><br />
		<?php
		$list = $this->check_other_xili_plugins();
		if ( '' != $list ) {
			?>
		<label for="xiliplugenable">
			<input type="checkbox" id="xiliplugenable" name="xiliplugenable" value="enable" <?php checked( $xiliplug, 'enable', true ); ?> />&nbsp;<?php echo 'Other xili plugins = ' . $list; ?>
		</label><br /><br />
		<?php } ?>
		<label for="webmestre"><?php esc_html_e( 'Type of webmaster:', 'xili-tidy-tags' ); ?>
		<select name="webmestre" id="webmestre" style="width:100%;">
		<?php
		if ( ! isset( $this->xili_settings['webmestre-level'] ) ) {
			$this->xili_settings['webmestre-level'] = '?';
		}
		?>
			<option value="?" <?php selected( $this->xili_settings['webmestre-level'], '?' ); ?>><?php esc_html_e( 'Define your experience as webmaster…', 'xili-tidy-tags' ); ?></option>
			<option value="newbie" <?php selected( $this->xili_settings['webmestre-level'], 'newbie' ); ?>><?php esc_html_e( 'Newbie in WP', 'xili-tidy-tags' ); ?></option>
			<option value="wp-php" <?php selected( $this->xili_settings['webmestre-level'], 'wp-php' ); ?>><?php esc_html_e( 'Good knowledge in WP and few in php', 'xili-tidy-tags' ); ?></option>
			<option value="wp-php-dev" <?php selected( $this->xili_settings['webmestre-level'], 'wp-php-dev' ); ?>><?php esc_html_e( 'Good knowledge in WP, CMS and good in php', 'xili-tidy-tags' ); ?></option>
			<option value="wp-plugin-theme" <?php selected( $this->xili_settings['webmestre-level'], 'wp-plugin-theme' ); ?>><?php esc_html_e( 'WP theme and /or plugin developper', 'xili-tidy-tags' ); ?></option>
		</select></label>
		<br /><br />
		<label for="subject"><?php esc_html_e( 'Subject:', 'xili-tidy-tags' ); ?>
		<input class="widefat" id="subject" name="subject" type="text" value='' /></label>
		<select name="thema" id="thema" style="width:100%;">
			<option value='' ><?php esc_html_e( 'Choose topic... ', 'xili-tidy-tags' ); ?></option>
			<option value="Message" ><?php esc_html_e( 'Message', 'xili-tidy-tags' ); ?></option>
			<option value="Question" ><?php esc_html_e( 'Question', 'xili-tidy-tags' ); ?></option>
			<option value="Encouragement" ><?php esc_html_e( 'Encouragement', 'xili-tidy-tags' ); ?></option>
			<option value="Support need" ><?php esc_html_e( 'Support need', 'xili-tidy-tags' ); ?></option>
		</select>
		<textarea class="widefat" rows="5" cols="20" id="mailcontent" name="mailcontent"><?php esc_html_e( 'Your message here…', 'xili-tidy-tags' ); ?></textarea>
		</fieldset>
		<p>
		<?php esc_html_e( 'Before send the mail, check the infos to be sent and complete textarea. A copy (Cc:) is sent to webmaster email (modify it if needed) . ', 'xili-tidy-tags' ); ?>
		</p>
		<div class='submit'>
		<input id='sendmail' name='sendmail' type='submit' tabindex='6' value="<?php esc_html_e( 'Send email', 'xili-tidy-tags' ); ?>" /></div>

		<div style="clear:both; height:1px"></div>
		<?php
	}

} // end of admin class
